@@ -31,6 +31,9 @@
 #include <sys/utsname.h>
 #endif
 #include <zlib.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <dirent.h>
 
 #include "shared/optparser.h"
 #include "shared/misc.h"
@@ -38,7 +41,9 @@
 #include "libclamav/str.h"
 #include "libclamav/clamav.h"
 #include "libclamav/others.h"
+#include "libclamav/readdb.h"
 #include "libclamav/bytecode.h"
+#include "libclamav/bytecode_detect.h"
 #include "target.h"
 
 #ifndef _WIN32
@@ -57,11 +62,6 @@ static struct _cfgfile {
     { NULL,		    0		    }
 };
 
-static const char *dbnames[] = { "main.cvd", "main.cld", "daily.cvd",
-				 "daily.cld", "safebrowsing.cvd",
-				 "safebrowsing.cld", "bytecode.cvd",
-				 "bytecode.cld", NULL };
-
 static void printopts(struct optstruct *opts, int nondef)
 {
 	const struct optstruct *opt;
@@ -211,17 +211,12 @@ static void help(void)
     return;
 }
 
-static void print_platform(void)
+static void print_platform(struct cli_environment *env)
 {
-#ifdef HAVE_UNAME_SYSCALL
-    struct utsname name;
-#endif
     printf("\nPlatform information\n--------------------\n");
-#ifdef HAVE_UNAME_SYSCALL
-    uname(&name);
-    printf("uname: %s %s %s %s\n", name.sysname, name.release, name.version,
-	   name.machine);
-#endif
+    printf("uname: %s %s %s %s\n",
+	   env->sysname, env->release, env->version, env->machine);
+
     printf("OS: "TARGET_OS_TYPE", ARCH: "TARGET_ARCH_TYPE", CPU: "TARGET_CPU_TYPE"\n");
 
 #ifdef C_LINUX
@@ -232,15 +227,6 @@ static void print_platform(void)
 	   perror("failed to determine");
 	}
     }
-#elif defined(_WIN32)
-    {
-	/* get just some basic information, since getting the full version
-	 * is too complicated */
-	uint32_t dwVersion = GetVersion();
-	printf("Full OS version: %d.%d (%d)\n",
-	   dwVersion&0xff, (dwVersion>>8)&0xff,
-	   dwVersion<0x80000000 ? dwVersion>>16 : 0);
-    }
 #else
     /* e.g. Solaris */
     if (!access("/etc/release", R_OK)) {
@@ -259,31 +245,114 @@ static void print_platform(void)
     printf("zlib version: %s (%s)\n",
 	   ZLIB_VERSION, zlibVersion());
 #endif
+    if (env->triple[0])
+    printf("Triple: %s\n", env->triple);
+    if (env->cpu[0])
+	printf("CPU: %s, %s\n", env->cpu, env->big_endian ? "Big-endian" : "Little-endian");
+    printf("platform id: 0x%08x%08x%08x\n",
+	   env->platform_id_a,
+	   env->platform_id_b,
+	   env->platform_id_c);
 }
 
-static void print_build(void)
+static void print_build(struct cli_environment *env)
 {
+    const char *name;
+    const char *version = NULL;
     printf("\nBuild information\n-----------------\n");
     /* Try to print information about some commonly used compilers */
 #ifdef __GNUC__
-    printf("GNU C: %s (%u.%u.%u)\n", __VERSION__, __GNUC__, __GNUC_MINOR__,
-	   __GNUC_PATCHLEVEL__);
-#endif
-#ifdef __INTEL_COMPILER
-    printf("Intel Compiler %u\n", __INTEL_COMPILER);
-#endif
-#ifdef _MSC_VER
-    printf("Microsoft Visual C++ %u\n", _MSC_VER);
-#endif
-#ifdef __SUNPRO_C
-    printf("Sun studio %u\n", __SUNPRO_C);
+	version = __VERSION__;
 #endif
+    switch (env->compiler) {
+	case compiler_gnuc:
+	    name = "GNU C";
+	    break;
+	case compiler_clang:
+	    name = "Clang";
+	    break;
+	case compiler_llvm:
+	    name = "LLVM-GCC";
+	    break;
+	case compiler_intel:
+	    name = "Intel Compiler";
+	    break;
+	case compiler_msc:
+	    name = "Microsoft Visual C++";
+	    break;
+	case compiler_sun:
+	    name = "Sun studio";
+	    break;
+	default:
+	    name = NULL;
+    }
+    if (name)
+	printf("%s: %s%s(%u.%u.%u)\n", name,
+	       version ? version : "",
+	       version ? " " : "",
+	       env->c_version >> 16,
+		   (env->c_version >> 8)&0xff,
+		   (env->c_version)&0xff);
     cli_printcxxver();
 #if defined(BUILD_CPPFLAGS) && defined(BUILD_CFLAGS) && defined(BUILD_CXXFLAGS) && defined(BUILD_LDFLAGS) && defined(BUILD_CONFIGURE_FLAGS)
     printf("CPPFLAGS: %s\nCFLAGS: %s\nCXXFLAGS: %s\nLDFLAGS: %s\nConfigure: %s\n",
 	   BUILD_CPPFLAGS, BUILD_CFLAGS, BUILD_CXXFLAGS, BUILD_LDFLAGS,
 	   BUILD_CONFIGURE_FLAGS);
 #endif
+    printf("sizeof(void*) = %d\n", env->sizeof_ptr);
+    printf("Engine flevel: %d, dconf: %d\n",
+	   env->functionality_level,
+	   env->dconf_level);
+}
+
+static void print_dbs(const char *dir)
+{
+	DIR *dd;
+	struct dirent *dent;
+	char *dbfile;
+	unsigned int flevel = cl_retflevel(), cnt, sigs = 0;
+	struct cl_cvd *cvd;
+
+    if((dd = opendir(dir)) == NULL) {
+        printf("print_dbs: Can't open directory %s\n", dir);
+        return;
+    }
+
+    while((dent = readdir(dd))) {
+	if(dent->d_ino) {
+	    if(CLI_DBEXT(dent->d_name)) {
+		dbfile = (char *) malloc(strlen(dent->d_name) + strlen(dir) + 2);
+		if(!dbfile) {
+		    printf("print_dbs: Can't allocate memory for dbfile\n");
+		    return;
+		}
+		sprintf(dbfile, "%s"PATHSEP"%s", dir, dent->d_name);
+		if(cli_strbcasestr(dbfile, ".cvd") || cli_strbcasestr(dbfile, ".cld")) {
+		    cvd = cl_cvdhead(dbfile);
+		    if(!cvd) {
+			printf("%s: Can't get information about the database\n", dbfile);
+		    } else {
+			const time_t t = cvd->stime;
+			printf("%s: version %u, sigs: %u, built on %s", dent->d_name, cvd->version, cvd->sigs, ctime(&t));
+			sigs += cvd->sigs;
+			if(cvd->fl > flevel)
+			    printf("%s: WARNING: This database requires f-level %u (current f-level: %u)\n", dent->d_name, cvd->fl, flevel);
+			cl_cvdfree(cvd);
+		    }
+		} else if(cli_strbcasestr(dbfile, ".cbc")) {
+		    printf("[3rd Party] %s: bytecode\n", dent->d_name);
+		    sigs++;
+		} else {
+		    cnt = countlines(dbfile);
+		    printf("[3rd Party] %s: %u sig%c\n", dent->d_name, cnt, cnt > 1 ? 's' : ' ');
+		    sigs += cnt;
+		}
+		free(dbfile);
+	    }
+	}
+    }
+    closedir(dd);
+    printf("Total number of signatures: %u\n", sigs);
 }
 
 int main(int argc, char **argv)
@@ -293,8 +362,7 @@ int main(int argc, char **argv)
 	struct optstruct *opts, *toolopts;
 	const struct optstruct *opt;
 	unsigned int i, j;
-	struct cl_cvd *cvd;
-	unsigned int flevel;
+	struct cli_environment env;
 
 
     opts = optparse(NULL, argc, argv, 1, OPT_CLAMCONF, 0, NULL);
@@ -395,27 +463,15 @@ int main(int argc, char **argv)
 	}
 	dbdir[sizeof(dbdir) - 1] = 0;
     }
+
+    printf("\nDatabase information\n--------------------\n");
     printf("Database directory: %s\n", dbdir);
     if(strcmp(dbdir, clamd_dbdir))
 	printf("WARNING: freshclam.conf and clamd.conf point to different database directories\n");
-    flevel = cl_retflevel();
-    for(i = 0; dbnames[i]; i++) {
-	snprintf(path, sizeof(path), "%s"PATHSEP"%s", dbdir, dbnames[i]);
-	path[511] = 0;
-	if(!access(path, R_OK)) {
-	    cvd = cl_cvdhead(path);
-	    if(!cvd) {
-		printf("%s: Can't get information about the database\n", dbnames[i]);
-	    } else {
-		const time_t t = cvd->stime;
-		printf("%s: version %u, sigs: %u, built on %s", dbnames[i], cvd->version, cvd->sigs, ctime(&t));
-		if(cvd->fl > flevel)
-		    printf("%s: WARNING: This database requires f-level %u (current f-level: %u)\n", dbnames[i], cvd->fl, flevel);
-		cl_cvdfree(cvd);
-	    }
-	}
-    }
-    print_platform();
-    print_build();
+    print_dbs(dbdir);
+
+    cli_detect_environment(&env);
+    print_platform(&env);
+    print_build(&env);
     return 0;
 }
