@@ -30,6 +30,7 @@
 #endif
 #ifndef _WIN32
 #include <sys/time.h>
+#include <sys/resource.h>
 #endif
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -42,6 +43,7 @@
 #include <grp.h>
 #endif
 #include <signal.h>
+#include <errno.h>
 
 #if defined(USE_SYSLOG) && !defined(C_AIX)
 #include <syslog.h>
@@ -107,6 +109,7 @@ int main(int argc, char **argv)
 #ifndef	_WIN32
         struct passwd *user = NULL;
 	struct sigaction sa;
+	struct rlimit rlim;
 #endif
 	time_t currtime;
 	const char *dbdir, *cfgfile;
@@ -294,6 +297,26 @@ int main(int argc, char **argv)
 	logg("#Running as user %s (UID %u, GID %u)\n", user->pw_name, user->pw_uid, user->pw_gid);
 #endif
 
+#ifdef RLIMIT_DATA
+    if (getrlimit(RLIMIT_DATA, &rlim) == 0) {
+       /* bb #1941.
+        * On 32-bit FreeBSD if you set ulimit -d to >2GB then mmap() will fail
+        * too soon (after ~120 MB).
+        * Set limit lower than 2G if on 32-bit */
+       uint64_t lim = rlim.rlim_cur;
+       lim = (int32_t) lim;
+       if (sizeof(void*) == 4 &&
+           lim != rlim.rlim_cur) {
+           rlim.rlim_cur = 2048*1024-1;
+           if (setrlimit(RLIMIT_DATA, &rlim) < 0)
+               logg("!setrlimit(RLIMIT_DATA) failed: %s\n", strerror(errno));
+           else
+               logg("^Running on 32-bit system, and RLIMIT_DATA > 2GB, lowering to 2GB!\n");
+       }
+    }
+#endif
+
+
     if(logg_size)
 	logg("#Log file size limited to %d bytes.\n", logg_size);
     else
@@ -431,6 +454,18 @@ int main(int argc, char **argv)
 	    break;
 	}
     }
+    if((opt = optget(opts,"BytecodeMode"))->enabled) {
+	enum bytecode_mode mode;
+	if (!strcmp(opt->strarg, "ForceJIT"))
+	    mode = CL_BYTECODE_MODE_JIT;
+	else if(!strcmp(opt->strarg, "ForceInterpreter"))
+	    mode = CL_BYTECODE_MODE_INTERPRETER;
+	else if(!strcmp(opt->strarg, "Test"))
+	    mode = CL_BYTECODE_MODE_TEST;
+	else
+	    mode = CL_BYTECODE_MODE_AUTO;
+	cl_engine_set_num(engine, CL_ENGINE_BYTECODE_MODE, mode);
+    }
     if((opt = optget(opts,"BytecodeTimeout"))->enabled) {
 	cl_engine_set_num(engine, CL_ENGINE_BYTECODE_TIMEOUT, opt->numarg);
     }
@@ -528,7 +563,7 @@ int main(int argc, char **argv)
 	gengine = engine;
 	atexit(free_engine);
 	if(daemonize() == -1) {
-	    logg("!daemonize() failed\n");
+	    logg("!daemonize() failed: %s\n", strerror(errno));
 	    ret = 1;
 	    break;
 	}
