@@ -77,6 +77,7 @@
 #include "libclamav/others.h"
 #include "libclamav/str.h"
 #include "libclamav/cvd.h"
+#include "libclamav/regex_list.h"
 
 extern char updtmpdir[512];
 
@@ -181,20 +182,24 @@ static int getclientsock(const char *localip, int prot)
     return socketfd;
 }
 
+static int qcompare(const void *a, const void *b)
+{
+    return (*(const struct addrinfo **) a)->ai_flags - (*(const struct addrinfo **) b)->ai_flags;
+}
+
 static int wwwconnect(const char *server, const char *proxy, int pport, char *ip, const char *localip, int ctimeout, struct mirdat *mdat, int logerr, unsigned int can_whitelist)
 {
 	int socketfd, port, ret;
-	unsigned int ips = 0, ignored = 0;
+	unsigned int ips = 0, ignored = 0, i;
 #ifdef HAVE_GETADDRINFO
-	struct addrinfo hints, *res = NULL, *rp, *loadbal_rp = NULL;
+	struct addrinfo hints, *res = NULL, *rp, *loadbal_rp = NULL, *addrs[128];
 	char port_s[6], loadbal_ipaddr[46];
-	uint32_t loadbal = 1, minsucc = 0xffffffff, minfail = 0xffffffff;
+	uint32_t loadbal = 1, minsucc = 0xffffffff, minfail = 0xffffffff, addrnum = 0;
 	struct mirdat_ip *md;
 #else
 	struct sockaddr_in name;
 	struct hostent *host;
 	unsigned char *ia;
-	int i;
 #endif
 	char ipaddr[46];
 	const char *hostpt;
@@ -237,9 +242,17 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
 	return -1;
     }
 
-    for(rp = res; rp; rp = rp->ai_next) {
+    for(rp = res; rp && addrnum < 128; rp = rp->ai_next) {
+	rp->ai_flags = cli_rndnum(1024);
+	addrs[addrnum] = rp;
+	addrnum++;
+    }
+    qsort(addrs, addrnum, sizeof(struct addrinfo *), qcompare);
+
+    for(i = 0; i < addrnum; i++) {
 	    void *addr;
 
+	rp = addrs[i];
 	ips++;
 #ifdef SUPPORT_IPv6
 	if(rp->ai_family == AF_INET6)
@@ -261,7 +274,7 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
 		logg("*Ignoring mirror %s (has connected too many times with an outdated version)\n", ipaddr);
 
 	    ignored++;
-	    if(!loadbal || rp->ai_next)
+	    if(!loadbal || i + 1 < addrnum)
 		continue;
 	}
 
@@ -277,15 +290,15 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
 			loadbal_rp = rp;
 			strncpy(loadbal_ipaddr, ipaddr, sizeof(loadbal_ipaddr));
 		    }
-		    if(rp->ai_next)
+		    if(i + 1 < addrnum)
 			continue;
 		}
 	    }
 
 	    if(!loadbal_rp) {
-		if(!rp->ai_next) {
+		if(i + 1 == addrnum) {
 		    loadbal = 0;
-		    rp = res;
+		    i = 0;
 		}
 		continue;
 	    }
@@ -299,7 +312,7 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
 	if(ip)
 	    strcpy(ip, ipaddr);
 
-	if(rp != res)
+	if(rp != loadbal_rp && rp != addrs[0])
 	    logg("Trying host %s (%s)...\n", hostpt, ipaddr);
 
 	socketfd = getclientsock(localip, rp->ai_family);
@@ -317,7 +330,7 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
 	    closesocket(socketfd);
 	    if(loadbal) {
 		loadbal = 0;
-		rp = res;
+		i = 0;
 	    }
 	    continue;
 	} else {
@@ -569,6 +582,7 @@ int submitstats(const char *clamdcfg, const struct optstruct *opts)
 
     if(!(opt = optget(clamdopt, "LogFile"))->enabled) {
 	logg("!SubmitDetectionStats: LogFile needs to be enabled in %s\n", clamdcfg);
+	logg("SubmitDetectionStats: Please consider enabling ExtendedDetectionInfo\n");
 	optfree(clamdopt);
 	return 56;
     }
@@ -683,13 +697,13 @@ int submitstats(const char *clamdcfg, const struct optstruct *opts)
 	pt2 = &pt[strlen(pt) - 6];
 	*pt2 = 0;
 
-	if(!(pt2 = strrchr(pt, ':'))) {
+	if(!(pt2 = strrchr(pt, ' ')) || pt2[-1] != ':') {
 	    logg("!SubmitDetectionStats: Incorrect format of the log file (1)\n");
 	    ret = 1;
 	    break;
 	}
-	*pt2 = 0;
-	pt2 += 2;
+	pt2[-1] = 0;
+	pt2++;
 
 	if((pt = strrchr(pt, *PATHSEP)))
 	    *pt++ = 0;
@@ -1754,7 +1768,7 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
 	    cl_engine_free(engine);
 	    return 55;
 	}
-	if((ret = cli_bytecode_prepare(&engine->bcs, engine->dconf->bytecode/*FIXME: dconf has no sense here*/))) {
+	if(optget(opts, "Bytecode")->enabled && (ret = cli_bytecode_prepare2(engine, &engine->bcs, engine->dconf->bytecode/*FIXME: dconf has no sense here*/))) {
 	    logg("!Failed to compile/load bytecode: %s\n", cl_strerror(ret));
 	    unlink(newfile);
 	    free(newfile);
@@ -1762,6 +1776,8 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
 	    return 55;
 	}
 	logg("*Properly loaded %u signatures from new %s\n", newsigs, newdb);
+	if(engine->domainlist_matcher && engine->domainlist_matcher->sha256_pfx_set.keys)
+	    cli_hashset_destroy(&engine->domainlist_matcher->sha256_pfx_set);
 	cl_engine_free(engine);
     }
 
@@ -2013,6 +2029,7 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
     }
 
     mirman_write("mirrors.dat", &mdat);
+    cli_rmdirs(updtmpdir);
 
     if(updated) {
 	if(optget(opts, "HTTPProxyServer")->enabled) {
@@ -2054,7 +2071,6 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
 		    free(cmd);
 		    if(newver)
 			free(newver);
-		    cli_rmdirs(updtmpdir);
 		    return 75;
 		}
 
@@ -2077,6 +2093,5 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
     if(newver)
 	free(newver);
 
-    cli_rmdirs(updtmpdir);
     return 0;
 }
