@@ -32,10 +32,12 @@
 #include "pe.h"
 #include "bytecode.h"
 #include "bytecode_priv.h"
+#include "bytecode_detect.h"
 #include "readdb.h"
 #include "scanners.h"
 #include "bytecode_api.h"
 #include "bytecode_api_impl.h"
+#include "builtin_bytecodes.h"
 #include <string.h>
 
 /* dummy values */
@@ -128,7 +130,6 @@ static int cli_bytecode_context_reset(struct cli_bc_ctx *ctx)
 
     if (ctx->outfd) {
 	cli_ctx *cctx = ctx->ctx;
-	cli_bcapi_extract_new(ctx, -1);
 	if (ctx->outfd)
 	    close(ctx->outfd);
 	if (ctx->tempfile && (!cctx || !cctx->engine->keeptmp)) {
@@ -147,10 +148,10 @@ static int cli_bytecode_context_reset(struct cli_bc_ctx *ctx)
 	    snprintf(fullname, 1024, "%s"PATHSEP"javascript", ctx->jsnormdir);
 	    fd = open(fullname, O_RDONLY|O_BINARY);
 	    if(fd >= 0) {
-		ret = cli_scandesc(fd, cctx, CL_TYPE_HTML, 0, NULL, AC_SCAN_VIR);
+		ret = cli_scandesc(fd, cctx, CL_TYPE_HTML, 0, NULL, AC_SCAN_VIR, NULL);
 		if (ret == CL_CLEAN) {
 		    lseek(fd, 0, SEEK_SET);
-		    ret = cli_scandesc(fd, cctx, CL_TYPE_TEXT_ASCII, 0, NULL, AC_SCAN_VIR);
+		    ret = cli_scandesc(fd, cctx, CL_TYPE_TEXT_ASCII, 0, NULL, AC_SCAN_VIR, NULL);
 		}
 		close(fd);
 	    }
@@ -213,7 +214,6 @@ static int cli_bytecode_context_reset(struct cli_bc_ctx *ctx)
 
 int cli_bytecode_context_clear(struct cli_bc_ctx *ctx)
 {
-    cli_ctx *cctx = (cli_ctx*)ctx->ctx;
     cli_bytecode_context_reset(ctx);
     memset(ctx, 0, sizeof(*ctx));
     return CL_SUCCESS;
@@ -221,6 +221,9 @@ int cli_bytecode_context_clear(struct cli_bc_ctx *ctx)
 
 static unsigned typesize(const struct cli_bc *bc, uint16_t type)
 {
+    struct cli_bc_type *ty;
+    unsigned j;
+
     type &= 0x7fff;
     if (!type)
 	return 0;
@@ -232,7 +235,25 @@ static unsigned typesize(const struct cli_bc *bc, uint16_t type)
 	return 4;
     if (type <= 64)
 	return 8;
-    return bc->types[type-65].size;
+    ty = &bc->types[type-65];
+    if (ty->size)
+	return ty->size;
+    switch (ty->kind) {
+	case 2:
+	case 3:
+	    for (j=0;j<ty->numElements;j++)
+		ty->size += typesize(bc, ty->containedTypes[j]);
+	    break;
+	case 4:
+	    ty->size = ty->numElements * typesize(bc, ty->containedTypes[0]);
+	    break;
+	default:
+	    break;
+    }
+    if (!ty->size && ty->kind != DFunctionType) {
+	cli_warnmsg("type %d size is 0\n", type-65);
+    }
+    return ty->size;
 }
 
 static unsigned typealign(const struct cli_bc *bc, uint16_t type)
@@ -264,6 +285,10 @@ int cli_bytecode_context_setfuncid(struct cli_bc_ctx *ctx, const struct cli_bc *
 	    return CL_EMEM;
 	}
 	ctx->opsizes = cli_malloc(sizeof(*ctx->opsizes)*func->numArgs);
+	if (!ctx->opsizes) {
+	    cli_errmsg("bytecode: error allocating memory for opsizes\n");
+	    return CL_EMEM;
+	}
 	for (i=0;i<func->numArgs;i++) {
 	    unsigned al = typealign(bc, func->types[i]);
 	    s = (s+al-1)&~(al-1);
@@ -742,7 +767,8 @@ static int parseTypes(struct cli_bc *bc, unsigned char *buffer)
 		    return CL_EMALFDB;
 		}
 		if (t == 5) {
-		    ty->size = ty->align = sizeof(void*);
+		    /* for interpreter, pointers 64-bit there */
+		    ty->size = ty->align = 8;
 		} else {
 		    ty->size = ty->numElements*typesize(bc, ty->containedTypes[0]);
 		    ty->align = typealign(bc, ty->containedTypes[0]);
@@ -1055,10 +1081,14 @@ static int parseFunctionHeader(struct cli_bc *bc, unsigned fn, unsigned char *bu
 	return CL_EMALFDB;
     }
     all_locals = func->numArgs + func->numLocals;
-    func->types = cli_calloc(all_locals, sizeof(*func->types));
-    if (!func->types) {
-	cli_errmsg("Out of memory allocating function arguments\n");
-	return CL_EMEM;
+    if (!all_locals) {
+	func->types = NULL;
+    } else {
+	func->types = cli_calloc(all_locals, sizeof(*func->types));
+	if (!func->types) {
+	    cli_errmsg("Out of memory allocating function arguments\n");
+	    return CL_EMEM;
+	}
     }
     for (i=0;i<all_locals;i++) {
 	func->types[i] = readNumber(buffer, &offset, len, &ok);
@@ -1188,10 +1218,14 @@ static int parseBB(struct cli_bc *bc, unsigned func, unsigned bb, unsigned char
 		if (ok) {
 		    inst.u.ops.numOps = numOp;
 		    inst.u.ops.opsizes=NULL;
-		    inst.u.ops.ops = cli_calloc(numOp, sizeof(*inst.u.ops.ops));
-		    if (!inst.u.ops.ops) {
-			cli_errmsg("Out of memory allocating operands\n");
-			return CL_EMEM;
+		    if (!numOp) {
+			inst.u.ops.ops = NULL;
+		    } else {
+			inst.u.ops.ops = cli_calloc(numOp, sizeof(*inst.u.ops.ops));
+			if (!inst.u.ops.ops) {
+			    cli_errmsg("Out of memory allocating operands\n");
+			    return CL_EMEM;
+			}
 		    }
 		    if (inst.opcode == OP_BC_CALL_DIRECT)
 			inst.u.ops.funcid = readFuncID(bc, buffer, &offset, len, &ok);
@@ -1357,7 +1391,8 @@ enum parse_state {
     PARSE_BC_LSIG,
     PARSE_MD_OPT_HEADER,
     PARSE_FUNC_HEADER,
-    PARSE_BB
+    PARSE_BB,
+    PARSE_SKIP
 };
 
 int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int trust)
@@ -1382,9 +1417,18 @@ int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int tru
     }
     cli_chomp(firstbuf);
     rc = parseHeader(bc, (unsigned char*)firstbuf, &linelength);
+    state = PARSE_BC_LSIG;
     if (rc == CL_BREAK) {
+	const char *len = strchr(firstbuf, ':');
 	bc->state = bc_skip;
-	return CL_SUCCESS;
+	if (!linelength) {
+	    linelength = len ? atoi(len+1) : 4096;
+	}
+	if (linelength < 4096)
+	    linelength = 4096;
+	cli_dbgmsg("line: %d\n", linelength);
+	state = PARSE_SKIP;
+	rc = CL_SUCCESS;
     }
     if (rc != CL_SUCCESS) {
 	cli_errmsg("Error at bytecode line %u\n", row);
@@ -1395,7 +1439,6 @@ int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int tru
 	cli_errmsg("Out of memory allocating line of length %u\n", linelength);
 	return CL_EMEM;
     }
-    state = PARSE_BC_LSIG;
     while (cli_dbgets(buffer, linelength, f, dbio) && !end) {
 	cli_chomp(buffer);
 	row++;
@@ -1404,8 +1447,8 @@ int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int tru
 		rc = parseLSig(bc, buffer);
 		if (rc == CL_BREAK) /* skip */ {
 		    bc->state = bc_skip;
-		    free(buffer);
-		    return CL_SUCCESS;
+		    state = PARSE_SKIP;
+		    continue;
 		}
 		if (rc != CL_SUCCESS) {
 		    cli_errmsg("Error at bytecode line %u\n", row);
@@ -1427,8 +1470,8 @@ int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int tru
 		rc = parseApis(bc, (unsigned char*)buffer);
 		if (rc == CL_BREAK) /* skip */ {
 		    bc->state = bc_skip;
-		    free(buffer);
-		    return CL_SUCCESS;
+		    state = PARSE_SKIP;
+		    continue;
 		}
 		if (rc != CL_SUCCESS) {
 		    cli_errmsg("Error at bytecode line %u\n", row);
@@ -1441,8 +1484,8 @@ int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int tru
 		rc = parseGlobals(bc, (unsigned char*)buffer);
 		if (rc == CL_BREAK) /* skip */ {
 		    bc->state = bc_skip;
-		    free(buffer);
-		    return CL_SUCCESS;
+		    state = PARSE_SKIP;
+		    continue;
 		}
 		if (rc != CL_SUCCESS) {
 		    cli_errmsg("Error at bytecode line %u\n", row);
@@ -1496,6 +1539,14 @@ int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int tru
 		    current_func++;
 		}
 		break;
+	    case PARSE_SKIP:
+		/* stop at S (source code), readdb.c knows how to skip this one
+		 * */
+		if (buffer[0] == 'S')
+		    end = 1;
+		/* noop parse, but we need to use dbgets with dynamic buffer,
+		 * otherwise we get 'Line too long for provided buffer' */
+		break;
 	}
     }
     free(buffer);
@@ -1510,6 +1561,7 @@ int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int tru
 
 int cli_bytecode_run(const struct cli_all_bc *bcs, const struct cli_bc *bc, struct cli_bc_ctx *ctx)
 {
+    int ret;
     struct cli_bc_inst inst;
     struct cli_bc_func func;
     if (!ctx || !ctx->bc || !ctx->func)
@@ -1520,6 +1572,11 @@ int cli_bytecode_run(const struct cli_all_bc *bcs, const struct cli_bc *bc, stru
 	cli_errmsg("bytecode has to be prepared either for interpreter or JIT!\n");
 	return CL_EARG;
     }
+    if (bc->state == bc_disabled) {
+	cli_dbgmsg("bytecode triggered but running bytecodes is disabled\n");
+	return CL_SUCCESS;
+    }
+    ctx->env = &bcs->env;
     context_safe(ctx);
     if (bc->state == bc_interp) {
 	memset(&func, 0, sizeof(func));
@@ -1538,10 +1595,16 @@ int cli_bytecode_run(const struct cli_all_bc *bcs, const struct cli_bc *bc, stru
 	inst.u.ops.ops = ctx->operands;
 	inst.u.ops.opsizes = ctx->opsizes;
 	cli_dbgmsg("Bytecode: executing in interpeter mode\n");
-	return cli_vm_execute(ctx->bc, ctx, &func, &inst);
+	ret = cli_vm_execute(ctx->bc, ctx, &func, &inst);
+    } else {
+	cli_dbgmsg("Bytecode: executing in JIT mode\n");
+	ret = cli_vm_execute_jit(bcs, ctx, &bc->funcs[ctx->funcid]);
     }
-    cli_dbgmsg("Bytecode: executing in JIT mode\n");
-    return cli_vm_execute_jit(bcs, ctx, &bc->funcs[ctx->funcid]);
+    /* need to be called here to catch any extracted but not yet scanned files
+     */
+    if (ctx->outfd)
+	cli_bcapi_extract_new(ctx, -1);
+    return ret;
 }
 
 uint64_t cli_bytecode_context_getresult_int(struct cli_bc_ctx *ctx)
@@ -1608,6 +1671,7 @@ void cli_bytecode_destroy(struct cli_bc *bc)
 	cli_bitset_free(bc->uses_apis);
     free(bc->lsig);
     free(bc->globalBytes);
+    memset(bc, 0, sizeof(*bc));
 }
 
 #define MAP(val) do { operand_t o = val; \
@@ -1659,6 +1723,39 @@ static inline int get_geptypesize(const struct cli_bc *bc, uint16_t tid)
   return typesize(bc, ty->containedTypes[0]);
 }
 
+static int calc_gepz(struct cli_bc *bc, struct cli_bc_func *func, uint16_t tid, operand_t op)
+{
+    unsigned off = 0, i;
+    uint32_t *gepoff;
+    const struct cli_bc_type *ty;
+    if (tid >= bc->num_types + 65) {
+	cli_errmsg("bytecode: typeid out of range %u >= %u\n", tid, bc->num_types);
+	return -1;
+    }
+    if (tid <= 65) {
+	cli_errmsg("bytecode: invalid type for gep (%u)\n", tid);
+	return -1;
+    }
+    ty = &bc->types[tid - 65];
+    if (ty->kind != DPointerType || ty->containedTypes[0] < 65) {
+	cli_errmsg("bytecode: invalid gep type, must be pointer to nonint: %u\n", tid);
+	return -1;
+    }
+    ty = &bc->types[ty->containedTypes[0] - 65];
+    if (ty->kind != DStructType && ty->kind != DPackedStructType)
+	return 0;
+    gepoff = (uint32_t*)&func->constants[op - func->numValues];
+    if (*gepoff >= ty->numElements) {
+	cli_errmsg("bytecode: gep offset out of range: %d >= %d\n",(uint32_t)*gepoff, ty->numElements);
+	return -1;
+    }
+    for (i=0;i<*gepoff;i++) {
+	off += typesize(bc, ty->containedTypes[i]);
+    }
+    *gepoff = off;
+    return 1;
+}
+
 static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 {
     unsigned i, j, k;
@@ -1755,6 +1852,7 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 	    map[j] = bcfunc->numBytes;
 	    /* printf("%d -> %d, %u\n", j, map[j], typesize(bc, ty)); */
 	    bcfunc->numBytes += typesize(bc, ty);
+	    /* TODO: don't allow size 0, it is always a bug! */
 	}
 	bcfunc->numBytes = (bcfunc->numBytes + 7)&~7;
 	for (j=0;j<bcfunc->numConstants;j++) {
@@ -1872,11 +1970,13 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 		    else
 			inst->interp_op = 5*(inst->interp_op/5)+3;
 		    MAP(inst->u.three[1]);
+		    if (calc_gepz(bc, bcfunc, inst->u.three[0], inst->u.three[2]) == -1)
+			return CL_EBYTECODE;
 		    MAP(inst->u.three[2]);
 		    break;
-		case OP_BC_GEPN:
-		    /*TODO */
-		    break;
+/*		case OP_BC_GEPN:
+		    *TODO 
+		    break;*/
 		case OP_BC_MEMSET:
 		case OP_BC_MEMCPY:
 		case OP_BC_MEMMOVE:
@@ -1903,7 +2003,7 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 		    MAPPTR(inst->u.unaryop);
 		    break;
 		default:
-		    cli_dbgmsg("Unhandled opcode: %d\n", inst->opcode);
+		    cli_warnmsg("Bytecode: unhandled opcode: %d\n", inst->opcode);
 		    return CL_EBYTECODE;
 	    }
 	}
@@ -1914,41 +2014,408 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
     return CL_SUCCESS;
 }
 
-int cli_bytecode_prepare(struct cli_all_bc *bcs, unsigned dconfmask)
+static int add_selfcheck(struct cli_all_bc *bcs)
 {
-    unsigned i, interp = 0;
+    struct cli_bc_func *func;
+    struct cli_bc_inst *inst;
+    struct cli_bc *bc;
+
+    bcs->all_bcs = cli_realloc2(bcs->all_bcs, sizeof(*bcs->all_bcs)*(bcs->count+1));
+    if (!bcs->all_bcs) {
+	cli_errmsg("cli_loadcbc: Can't allocate memory for bytecode entry\n");
+	return CL_EMEM;
+    }
+    bc = &bcs->all_bcs[bcs->count++];
+    memset(bc, 0, sizeof(*bc));
+
+    bc->trusted = 1;
+    bc->num_globals = 1;
+    bc->globals = cli_calloc(1, sizeof(*bc->globals));
+    if (!bc->globals) {
+	cli_errmsg("Failed to allocate memory for globals\n");
+	return CL_EMEM;
+    }
+    bc->globals[0] = cli_calloc(1, sizeof(*bc->globals[0]));
+    if (!bc->globals[0]) {
+	cli_errmsg("Failed to allocate memory for globals\n");
+	return CL_EMEM;
+    }
+    bc->globaltys = cli_calloc(1, sizeof(*bc->globaltys));
+    if (!bc->globaltys) {
+	cli_errmsg("Failed to allocate memory for globaltypes\n");
+	return CL_EMEM;
+    }
+    bc->globaltys[0] = 32;
+    *bc->globals[0] = 0;
+    bc->id = ~0;
+    bc->kind = 0;
+    bc->num_types = 5;
+    bc->num_func = 1;
+    bc->funcs = cli_calloc(1, sizeof(*bc->funcs));
+    if (!bc->funcs) {
+	cli_errmsg("Failed to allocate memory for func\n");
+	return CL_EMEM;
+    }
+    func = bc->funcs;
+    func->numInsts = 2;
+    func->numLocals = 1;
+    func->numValues = 1;
+    func->numConstants = 1;
+    func->numBB = 1;
+    func->returnType = 32;
+    func->types = cli_calloc(1, sizeof(*func->types));
+    if (!func->types) {
+	cli_errmsg("Failed to allocate memory for types\n");
+	return CL_EMEM;
+    }
+    func->types[0] = 32;
+    func->BB = cli_calloc(1, sizeof(*func->BB));
+    if (!func->BB) {
+	cli_errmsg("Failed to allocate memory for BB\n");
+	return CL_EMEM;
+    }
+    func->allinsts = cli_calloc(2, sizeof(*func->allinsts));
+    if (!func->allinsts) {
+	cli_errmsg("Failed to allocate memory for insts\n");
+	return CL_EMEM;
+    }
+    func->BB->numInsts = 2;
+    func->BB->insts = func->allinsts;
+    func->constants = cli_calloc(1, sizeof(*func->constants));
+    if (!func->constants) {
+	cli_errmsg("Failed to allocate memory for constants\n");
+	return CL_EMEM;
+    }
+    func->constants[0] = 0xf00d;
+    inst = func->allinsts;
+
+    inst->opcode = OP_BC_CALL_API;
+    inst->u.ops.numOps = 1;
+    inst->u.ops.opsizes = NULL;
+    inst->u.ops.ops = cli_calloc(1, sizeof(*inst->u.ops.ops));
+    if (!inst->u.ops.ops) {
+	cli_errmsg("Failed to allocate memory for instructions\n");
+	return CL_EMEM;
+    }
+    inst->u.ops.ops[0] = 1;
+    inst->u.ops.funcid = 18; /* test2 */
+    inst->dest = 0;
+    inst->type = 32;
+    inst->interp_op = inst->opcode* 5 + 3;
+
+    inst = &func->allinsts[1];
+    inst->opcode = OP_BC_RET;
+    inst->type = 32;
+    inst->u.unaryop = 0;
+    inst->interp_op = inst->opcode* 5;
+
+    bc->state = bc_loaded;
+    return 0;
+}
+
+static int run_selfcheck(struct cli_all_bc *bcs)
+{
+    struct cli_bc_ctx *ctx;
+    struct cli_bc *bc = &bcs->all_bcs[bcs->count-1];
     int rc;
-    if (cli_bytecode_prepare_jit(bcs) == CL_SUCCESS) {
-	cli_dbgmsg("Bytecode: %u bytecode prepared with JIT\n", bcs->count);
+    if (bc->state != bc_jit && bc->state != bc_interp) {
+	cli_errmsg("Failed to prepare selfcheck bytecode\n");
+	return CL_EBYTECODE;
+    }
+    ctx = cli_bytecode_context_alloc();
+    if (!ctx) {
+	cli_errmsg("Failed to allocate bytecode context\n");
+	return CL_EMEM;
+    }
+    cli_bytecode_context_setfuncid(ctx, bc, 0);
+
+    cli_dbgmsg("bytecode self test running\n");
+    ctx->bytecode_timeout = 0;
+    rc = cli_bytecode_run(bcs, bc, ctx);
+    cli_bytecode_context_destroy(ctx);
+    if (rc != CL_SUCCESS) {
+	cli_errmsg("bytecode self test failed: %s\n",
+		   cl_strerror(rc));
+    } else {
+	cli_dbgmsg("bytecode self test succeeded\n");
+    }
+    return rc;
+}
+
+static int selfcheck(int jit, struct cli_bcengine *engine)
+{
+    struct cli_all_bc bcs;
+    int rc;
+
+    memset(&bcs, 0, sizeof(bcs));
+    bcs.all_bcs = NULL;
+    bcs.count = 0;
+    bcs.engine = engine;
+    rc = add_selfcheck(&bcs);
+    if (rc == CL_SUCCESS) {
+	if (jit) {
+	    if (!bcs.engine) {
+		cli_dbgmsg("bytecode: JIT disabled\n");
+		rc = CL_BREAK;/* no JIT - not fatal */
+	    } else {
+		rc = cli_bytecode_prepare_jit(&bcs);
+	    }
+	} else {
+	    rc = cli_bytecode_prepare_interpreter(bcs.all_bcs);
+	}
+	if (rc == CL_SUCCESS)
+	    rc = run_selfcheck(&bcs);
+	if (rc == CL_BREAK)
+	    rc = CL_SUCCESS;
+    }
+    cli_bytecode_destroy(bcs.all_bcs);
+    free(bcs.all_bcs);
+    cli_bytecode_done_jit(&bcs, 1);
+    if (rc != CL_SUCCESS) {
+	cli_errmsg("Bytecode: failed to run selfcheck in %s mode: %s\n",
+		   jit ? "JIT" : "interpreter", cl_strerror(rc));
+    }
+    return rc;
+}
+
+static int set_mode(struct cl_engine *engine, enum bytecode_mode mode)
+{
+    if (engine->bytecode_mode == mode)
+	return 0;
+    if (engine->bytecode_mode == CL_BYTECODE_MODE_OFF) {
+	cli_errmsg("bytecode: already turned off, can't turn it on again!\n");
+	return -1;
+    }
+    cli_dbgmsg("Bytecode: mode changed to %d\n", mode);
+    if (engine->bytecode_mode == CL_BYTECODE_MODE_TEST) {
+	if (mode == CL_BYTECODE_MODE_OFF || have_clamjit) {
+	    cli_errmsg("bytecode: in test mode but JIT/bytecode is about to be disabled: %d\n", mode);
+	    engine->bytecode_mode = mode;
+	    return -1;
+	}
+	return 0;
+    }
+    if (engine->bytecode_mode == CL_BYTECODE_MODE_JIT) {
+	cli_errmsg("bytecode: in JIT mode but JIT is about to be disabled: %d\n", mode);
+	engine->bytecode_mode = mode;
+	return -1;
+    }
+    engine->bytecode_mode = mode;
+    return 0;
+}
+
+/* runs the first bytecode of the specified kind, or the builtin one if no
+ * bytecode of that kind is loaded */
+static int run_builtin_or_loaded(struct cli_all_bc *bcs, uint8_t kind, const char* builtin_cbc, struct cli_bc_ctx *ctx, const char *desc)
+{
+    unsigned i, builtin = 0, rc = 0;
+    struct cli_bc *bc = NULL;
+
+    for (i=0;i<bcs->count;i++) {
+	bc = &bcs->all_bcs[i];
+	if (bc->kind == kind)
+	    break;
+    }
+    if (i == bcs->count)
+	bc = NULL;
+    if (!bc) {
+	/* no loaded bytecode found, load the builtin one! */
+	struct cli_dbio dbio;
+	bc = cli_calloc(1, sizeof(*bc));
+	if (!bc) {
+	    cli_errmsg("Out of memory allocating bytecode\n");
+	    return CL_EMEM;
+	}
+	builtin = 1;
+
+	memset(&dbio, 0, sizeof(dbio));
+	dbio.usebuf = 1;
+	dbio.bufpt = dbio.buf = (char*)builtin_cbc;
+	dbio.bufsize = strlen(builtin_cbc)+1;
+	if (!dbio.bufsize || dbio.bufpt[dbio.bufsize-2] != '\n') {
+	    cli_errmsg("Invalid builtin bytecode: missing terminator\n");
+	    free(bc);
+	    return CL_EMALFDB;
+	}
+
+	rc = cli_bytecode_load(bc, NULL, &dbio, 1);
+	if (rc) {
+	    cli_errmsg("Failed to load builtin %s bytecode\n", desc);
+	    free(bc);
+	    return rc;
+	}
+    }
+    rc = cli_bytecode_prepare_interpreter(bc);
+    if (rc) {
+	cli_errmsg("Failed to prepare %s %s bytecode for interpreter: %s\n",
+		   builtin ? "builtin" : "loaded", desc, cl_strerror(rc));
+    }
+    if (bc->state != bc_interp) {
+	cli_errmsg("Failed to prepare %s %s bytecode for interpreter\n",
+		   builtin ? "builtin" : "loaded", desc);
+	rc = CL_EMALFDB;
+    }
+    if (!rc) {
+	cli_bytecode_context_setfuncid(ctx, bc, 0);
+	cli_dbgmsg("Bytecode: %s running (%s)\n", desc,
+		   builtin ? "builtin" : "loaded");
+	rc = cli_bytecode_run(bcs, bc, ctx);
+    }
+    if (rc) {
+	cli_errmsg("Failed to execute %s %s bytecode: %s\n",builtin ? "builtin":"loaded",
+		   desc, cl_strerror(rc));
+    }
+    if (builtin) {
+	cli_bytecode_destroy(bc);
+	free(bc);
+    }
+    return rc;
+}
+
+int cli_bytecode_prepare2(struct cl_engine *engine, struct cli_all_bc *bcs, unsigned dconfmask)
+{
+    unsigned i, interp = 0, jitok = 0, jitcount=0;
+    int rc;
+    struct cli_bc_ctx *ctx;
+
+    cli_detect_environment(&bcs->env);
+    switch (bcs->env.arch) {
+	case arch_i386:
+	case arch_x86_64:
+	    if (!(dconfmask & BYTECODE_JIT_X86)) {
+		cli_dbgmsg("Bytecode: disabled on X86 via DCONF\n");
+		if (set_mode(engine, CL_BYTECODE_MODE_INTERPRETER) == -1)
+		    return CL_EBYTECODE_TESTFAIL;
+	    }
+	    break;
+	case arch_ppc32:
+	case arch_ppc64:
+	    if (!(dconfmask & BYTECODE_JIT_PPC)) {
+		cli_dbgmsg("Bytecode: disabled on PPC via DCONF\n");
+		if (set_mode(engine, CL_BYTECODE_MODE_INTERPRETER) == -1)
+		    return CL_EBYTECODE_TESTFAIL;
+	    }
+	    break;
+	case arch_arm:
+	    if (!(dconfmask & BYTECODE_JIT_ARM)) {
+		cli_dbgmsg("Bytecode: disabled on ARM via DCONF\n");
+		if (set_mode(engine, CL_BYTECODE_MODE_INTERPRETER) == -1)
+		    return CL_EBYTECODE_TESTFAIL;
+	    }
+	    break;
+	default:
+	    cli_dbgmsg("Bytecode: JIT not supported on this architecture, falling back\n");
+	    if (set_mode(engine, CL_BYTECODE_MODE_INTERPRETER) == -1)
+		return CL_EBYTECODE_TESTFAIL;
+	    break;
+    }
+    cli_dbgmsg("Bytecode: mode is %d\n", engine->bytecode_mode);
+
+    ctx = cli_bytecode_context_alloc();
+    if (!ctx) {
+	cli_errmsg("Bytecode: failed to allocate bytecode context\n");
+	return CL_EMEM;
+    }
+    rc = run_builtin_or_loaded(bcs, BC_STARTUP, builtin_bc_startup, ctx, "BC_STARTUP");
+    if (rc != CL_SUCCESS) {
+	cli_warnmsg("Bytecode: BC_STARTUP failed to run, disabling ALL bytecodes! Please report to http://bugs.clamav.net\n");
+	ctx->bytecode_disable_status = 2;
+    } else {
+	cli_dbgmsg("Bytecode: disable status is %d\n", ctx->bytecode_disable_status);
+	rc = cli_bytecode_context_getresult_int(ctx);
+	/* check magic number, don't use 0 here because it is too easy for a
+	 * buggy bytecode to return 0 */
+	if (rc != 0xda7aba5e) {
+	    cli_warnmsg("Bytecode: selftest failed with code %08x. Please report to http://bugs.clamav.net\n",
+			rc);
+	    if (engine->bytecode_mode == CL_BYTECODE_MODE_TEST)
+		return CL_EBYTECODE_TESTFAIL;
+	}
+    }
+    switch (ctx->bytecode_disable_status) {
+	case 1:
+	    if (set_mode(engine, CL_BYTECODE_MODE_INTERPRETER) == -1)
+		return CL_EBYTECODE_TESTFAIL;
+	    break;
+	case 2:
+	    if (set_mode(engine, CL_BYTECODE_MODE_OFF) == -1)
+		return CL_EBYTECODE_TESTFAIL;
+	    break;
+	default:
+	    break;
+    }
+    cli_bytecode_context_destroy(ctx);
+
+
+    if (engine->bytecode_mode != CL_BYTECODE_MODE_INTERPRETER &&
+	engine->bytecode_mode != CL_BYTECODE_MODE_OFF) {
+	selfcheck(1, bcs->engine);
+	rc = cli_bytecode_prepare_jit(bcs);
+	if (rc == CL_SUCCESS) {
+	    jitok = 1;
+	    cli_dbgmsg("Bytecode: %u bytecode prepared with JIT\n", bcs->count);
+	    if (engine->bytecode_mode != CL_BYTECODE_MODE_TEST)
+		return CL_SUCCESS;
+	}
+	if (engine->bytecode_mode == CL_BYTECODE_MODE_JIT) {
+	    cli_errmsg("Bytecode: JIT required, but not all bytecodes could be prepared with JIT\n");
+	    return CL_EMALFDB;
+	}
+    } else {
+	cli_bytecode_done_jit(bcs, 0);
+    }
+
+    if (!(dconfmask & BYTECODE_INTERPRETER)) {
+	cli_dbgmsg("Bytecode: needs interpreter, but interpreter is disabled\n");
+	if (set_mode(engine, CL_BYTECODE_MODE_OFF) == -1)
+	    return CL_EBYTECODE_TESTFAIL;
+    }
+
+    if (engine->bytecode_mode == CL_BYTECODE_MODE_OFF) {
+	for (i=0;i<bcs->count;i++)
+	    bcs->all_bcs[i].state = bc_disabled;
+	cli_dbgmsg("Bytecode: ALL bytecodes disabled\n");
 	return CL_SUCCESS;
     }
+
     for (i=0;i<bcs->count;i++) {
 	struct cli_bc *bc = &bcs->all_bcs[i];
-	if (bc->state == bc_interp || bc->state == bc_jit)
+	if (bc->state == bc_jit) {
+	    jitcount++;
 	    continue;
-	if (!(dconfmask & BYTECODE_INTERPRETER)) {
-	    cli_warnmsg("Bytecode needs interpreter, but interpreter is disabled\n");
+	}
+	if (bc->state == bc_interp) {
+	    interp++;
 	    continue;
 	}
 	rc = cli_bytecode_prepare_interpreter(bc);
-	interp++;
-	if (rc != CL_SUCCESS)
+	if (rc != CL_SUCCESS) {
+	    bc->state = bc_disabled;
+	    cli_warnmsg("Bytecode: %d failed to prepare for interpreter mode\n", bc->id);
 	    return rc;
+	}
+	interp++;
     }
     cli_dbgmsg("Bytecode: %u bytecode prepared with JIT, "
-	       "%u prepared with interpreter\n", bcs->count-interp, interp);
+	       "%u prepared with interpreter, %u failed\n", jitcount, interp,
+	       bcs->count - jitcount - interp);
     return CL_SUCCESS;
 }
 
-int cli_bytecode_init(struct cli_all_bc *allbc, unsigned dconfmask)
+int cli_bytecode_init(struct cli_all_bc *allbc)
 {
+    int ret;
     memset(allbc, 0, sizeof(*allbc));
-    return cli_bytecode_init_jit(allbc, dconfmask);
+    ret = cli_bytecode_init_jit(allbc, 0/*XXX*/);
+    cli_dbgmsg("Bytecode initialized in %s mode\n",
+	       allbc->engine ? "JIT" : "interpreter");
+    allbc->inited = 1;
+    return ret;
 }
 
 int cli_bytecode_done(struct cli_all_bc *allbc)
 {
-    return cli_bytecode_done_jit(allbc);
+    return cli_bytecode_done_jit(allbc, 0);
 }
 
 int cli_bytecode_context_setfile(struct cli_bc_ctx *ctx, fmap_t *map)
@@ -1959,26 +2426,43 @@ int cli_bytecode_context_setfile(struct cli_bc_ctx *ctx, fmap_t *map)
     return 0;
 }
 
-int cli_bytecode_runlsig(cli_ctx *cctx, const struct cli_all_bc *bcs, unsigned bc_idx, const char **virname, const uint32_t* lsigcnt, const uint32_t *lsigsuboff, fmap_t *map)
+int cli_bytecode_runlsig(cli_ctx *cctx, struct cli_target_info *tinfo,
+			 const struct cli_all_bc *bcs, unsigned bc_idx,
+			 const char **virname, const uint32_t* lsigcnt,
+			 const uint32_t *lsigsuboff, fmap_t *map)
 {
     int ret;
     struct cli_bc_ctx ctx;
     const struct cli_bc *bc = &bcs->all_bcs[bc_idx-1];
+    struct cli_pe_hook_data pehookdata;
 
-    if (bc->hook_lsig_id) {
-	cli_dbgmsg("hook lsig id %d matched (bc %d)\n", bc->hook_lsig_id, bc->id);
-	/* this is a bytecode for a hook, defer running it until hook is
-	 * executed, so that it has all the info for the hook */
-	if (cctx->hook_lsig_matches)
-	    cli_bitset_set(cctx->hook_lsig_matches, bc->hook_lsig_id-1);
-	return CL_SUCCESS;
-    }
     memset(&ctx, 0, sizeof(ctx));
     cli_bytecode_context_setfuncid(&ctx, bc, 0);
     ctx.hooks.match_counts = lsigcnt;
     ctx.hooks.match_offsets = lsigsuboff;
     cli_bytecode_context_setctx(&ctx, cctx);
     cli_bytecode_context_setfile(&ctx, map);
+    if (tinfo && tinfo->status == 1) {
+	ctx.sections = tinfo->exeinfo.section;
+	memset(&pehookdata, 0, sizeof(pehookdata));
+	pehookdata.offset = tinfo->exeinfo.offset;
+	pehookdata.ep = tinfo->exeinfo.ep;
+	pehookdata.nsections = tinfo->exeinfo.nsections;
+	pehookdata.hdr_size = tinfo->exeinfo.hdr_size;
+	ctx.hooks.pedata = &pehookdata;
+	ctx.resaddr = tinfo->exeinfo.res_addr;
+    }
+    if (bc->hook_lsig_id) {
+	cli_dbgmsg("hook lsig id %d matched (bc %d)\n", bc->hook_lsig_id, bc->id);
+	/* this is a bytecode for a hook, defer running it until hook is
+	 * executed, so that it has all the info for the hook */
+	if (cctx->hook_lsig_matches)
+	    cli_bitset_set(cctx->hook_lsig_matches, bc->hook_lsig_id-1);
+	/* save match counts */
+	memcpy(&ctx.lsigcnt, lsigcnt, 64*4);
+	memcpy(&ctx.lsigoff, lsigsuboff, 64*4);
+	return CL_SUCCESS;
+    }
 
     cli_dbgmsg("Running bytecode for logical signature match\n");
     ret = cli_bytecode_run(bcs, bc, &ctx);
@@ -1987,11 +2471,16 @@ int cli_bytecode_runlsig(cli_ctx *cctx, const struct cli_all_bc *bcs, unsigned b
 	return CL_SUCCESS;
     }
     if (ctx.virname) {
+	int rc;
 	cli_dbgmsg("Bytecode found virus: %s\n", ctx.virname);
 	if (virname)
 	    *virname = ctx.virname;
+	if (!strncmp(*virname, "BC.Heuristics", 13))
+	    rc = cli_found_possibly_unwanted(cctx);
+	else
+	    rc = CL_VIRUS;
 	cli_bytecode_context_clear(&ctx);
-	return CL_VIRUS;
+	return rc;
     }
     ret = cli_bytecode_context_getresult_int(&ctx);
     cli_dbgmsg("Bytecode %u returned code: %u\n", bc->id, ret);
@@ -2005,10 +2494,13 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
     const unsigned *hooks = engine->hooks[id - _BC_START_HOOKS];
     unsigned i, hooks_cnt = engine->hooks_cnt[id - _BC_START_HOOKS];
     int ret;
-    unsigned executed = 0;
+    unsigned executed = 0, breakflag = 0;
 
     cli_bytecode_context_setfile(ctx, map);
     cli_dbgmsg("Bytecode executing hook id %u (%u hooks)\n", id, hooks_cnt);
+    /* restore match counts */
+    ctx->hooks.match_counts = ctx->lsigcnt;
+    ctx->hooks.match_offsets = ctx->lsigoff;
     for (i=0;i < hooks_cnt;i++) {
 	const struct cli_bc *bc = &engine->bcs.all_bcs[hooks[i]];
 	if (bc->lsig) {
@@ -2034,6 +2526,10 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
 	ret = cli_bytecode_context_getresult_int(ctx);
 	/* TODO: use prefix here */
 	cli_dbgmsg("Bytecode %u returned %u\n", bc->id, ret);
+	if (ret == 0xcea5e) {
+	    cli_dbgmsg("Bytecode set BREAK flag in hook!\n");
+	    breakflag = 1;
+	}
 	if (!ret) {
 	    char *tempfile;
 	    int fd = cli_bytecode_context_getresult_file(ctx, &tempfile);
@@ -2071,7 +2567,7 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
 	cli_dbgmsg("Bytecode: executed %u bytecodes for this hook\n", executed);
     else
 	cli_dbgmsg("Bytecode: no logical signature matched, no bytecode executed\n");
-    return CL_CLEAN;
+    return breakflag ? CL_BREAK : CL_CLEAN;
 }
 
 int cli_bytecode_context_setpe(struct cli_bc_ctx *ctx, const struct cli_pe_hook_data *data, const struct cli_exe_section *sections)
@@ -2104,7 +2600,8 @@ void cli_bytecode_describe(const struct cli_bc *bc)
     printf("Bytecode format functionality level: %u\n", bc->metadata.formatlevel);
     printf("Bytecode metadata:\n\tcompiler version: %s\n",
 	   bc->metadata.compiler ? bc->metadata.compiler : "N/A");
-    printf("\tcompiled on: %s",
+    printf("\tcompiled on: (%d) %s",
+	   (uint32_t)stamp,
 	   cli_ctime(&stamp, buf, sizeof(buf)));
     printf("\tcompiled by: %s\n", bc->metadata.sigmaker ? bc->metadata.sigmaker : "N/A");
     /*TODO: parse and display arch name, also take it into account when
@@ -2115,6 +2612,9 @@ void cli_bytecode_describe(const struct cli_bc *bc)
 	case BC_GENERIC:
 	    puts("generic, not loadable by clamscan/clamd");
 	    break;
+	case BC_STARTUP:
+	    puts("run on startup (unique)");
+	    break;
 	case BC_LOGICAL:
 	    puts("logical only");
 	    break;
@@ -2142,6 +2642,15 @@ void cli_bytecode_describe(const struct cli_bc *bc)
 	    puts("files matching logical signature");
 	    break;
 	case BC_PE_UNPACKER:
+	    if (bc->lsig)
+		puts("PE files matching logical signature (unpacked)");
+	    else
+		puts("all PE files! (unpacked)");
+	    break;
+	case BC_PDF:
+	    puts("PDF files");
+	    break;
+	case BC_PE_ALL:
 	    if (bc->lsig)
 		puts("PE files matching logical signature");
 	    else
