@@ -196,6 +196,11 @@ static int64_t rtlib_sra_i64(int64_t a, int32_t b)
     return CLI_SRS(a, b);//CLI_./..
 }
 
+static void rtlib_bzero(void *s, size_t n)
+{
+    memset(s, 0, n);
+}
+
 // Resolve integer libcalls, but nothing else.
 static void* noUnknownFunctions(const std::string& name) {
     void *addr =
@@ -208,6 +213,7 @@ static void* noUnknownFunctions(const std::string& name) {
 	.Case("__ashrdi3", (void*)(intptr_t)rtlib_sra_i64)
 	.Case("__ashldi3", (void*)(intptr_t)rtlib_shl_i64)
 	.Case("__lshrdi3", (void*)(intptr_t)rtlib_srl_i64)
+	.Case("__bzero", (void*)(intptr_t)rtlib_bzero)
 	.Case("memmove", (void*)(intptr_t)memmove)
 	.Case("memcpy", (void*)(intptr_t)memcpy)
 	.Case("memset", (void*)(intptr_t)memset)
@@ -808,9 +814,12 @@ public:
 	    // Have an alloca -> some instruction uses its address otherwise
 	    // mem2reg would have converted it to an SSA register.
 	    // Enable stack protector for this function.
-	    F->addFnAttr(Attribute::StackProtect);
 	    F->addFnAttr(Attribute::StackProtectReq);
 	}
+	// always add stackprotect attribute (bb #2239), so we know this
+	// function was verified. If there is no alloca it won't actually add
+	// stack protector in emitted code so this won't slow down the app.
+	F->addFnAttr(Attribute::StackProtect);
     }
 
     Value *GEPOperand(Value *V) {
@@ -899,6 +908,10 @@ public:
 					   BytecodeID+"f"+Twine(j), M);
 	    Functions[j]->setDoesNotThrow();
 	    Functions[j]->setCallingConv(CallingConv::Fast);
+#ifdef C_LINUX
+	    /* bb #2270, this should really be fixed either by LLVM or GCC.*/
+	    Functions[j]->addFnAttr(Attribute::constructStackAlignmentFromInt(16));
+#endif
 	}
 	const Type *I32Ty = Type::getInt32Ty(Context);
 	PM.add(createDeadCodeEliminationPass());
@@ -1634,11 +1647,15 @@ int cli_vm_execute_jit(const struct cli_all_bc *bcs, struct cli_bc_ctx *ctx,
 	0
     };
 
-    if ((ret = pthread_create(&thread, NULL, bytecode_watchdog, &w))) {
-	errs() << "Bytecode: failed to create new thread!";
-	errs() << cli_strerror(ret, buf, sizeof(buf));
-	errs() << "\n";
-	return CL_EBYTECODE;
+    if (ctx->bytecode_timeout) {
+	/* only spawn if timeout is set.
+	 * we don't set timeout for selfcheck (see bb #2235) */
+	if ((ret = pthread_create(&thread, NULL, bytecode_watchdog, &w))) {
+	    errs() << "Bytecode: failed to create new thread!";
+	    errs() << cli_strerror(ret, buf, sizeof(buf));
+	    errs() << "\n";
+	    return CL_EBYTECODE;
+	}
     }
 
     ret = bytecode_execute((intptr_t)code, ctx);
@@ -1646,7 +1663,9 @@ int cli_vm_execute_jit(const struct cli_all_bc *bcs, struct cli_bc_ctx *ctx,
     w.finished = 1;
     pthread_cond_signal(&w.cond);
     pthread_mutex_unlock(&w.mutex);
-    pthread_join(thread, NULL);
+    if (ctx->bytecode_timeout) {
+	pthread_join(thread, NULL);
+    }
 
     if (cli_debug_flag) {
 	gettimeofday(&tv1, NULL);
@@ -1800,7 +1819,7 @@ int cli_bytecode_prepare_jit(struct cli_all_bc *bcs)
 
 	for (unsigned i=0;i<bcs->count;i++) {
 	    const struct cli_bc *bc = &bcs->all_bcs[i];
-	    if (bc->state == bc_skip)
+	    if (bc->state == bc_skip || bc->state == bc_interp)
 		continue;
 	    LLVMCodegen Codegen(bc, M, &CF, bcs->engine->compiledFunctions, EE,
 				OurFPM, apiFuncs, apiMap);
@@ -1821,7 +1840,7 @@ int cli_bytecode_prepare_jit(struct cli_all_bc *bcs)
 	}
 	delete [] apiFuncs;
     }
-    return -1;
+    return CL_SUCCESS;
   } catch (std::bad_alloc &badalloc) {
       errs() << MODULE << badalloc.what() << "\n";
       return CL_EMEM;
@@ -1872,63 +1891,6 @@ int bytecode_init(void)
 int cli_bytecode_init_jit(struct cli_all_bc *bcs, unsigned dconfmask)
 {
     LLVMApiScopedLock scopedLock;
-    bcs->engine = NULL;
-    Triple triple(sys::getHostTriple());
-    if (cli_debug_flag)
-	errs() << "host triple is: " << sys::getHostTriple() << "\n";
-    enum Triple::ArchType arch = triple.getArch();
-    switch (arch) {
-	case Triple::arm:
-	    if (!(dconfmask & BYTECODE_JIT_ARM)) {
-		if (cli_debug_flag)
-		    errs() << "host triple is: " << sys::getHostTriple() << "\n";
-		return 0;
-	    }
-	    break;
-	case Triple::ppc:
-	case Triple::ppc64:
-	    if (!(dconfmask & BYTECODE_JIT_PPC)) {
-		if (cli_debug_flag)
-		    errs() << "JIT disabled for ppc\n";
-		return 0;
-	    }
-	    break;
-	case Triple::x86:
-	case Triple::x86_64:
-	    if (!(dconfmask & BYTECODE_JIT_X86)) {
-		if (cli_debug_flag)
-		    errs() << "JIT disabled for x86\n";
-		return 0;
-	    }
-	    break;
-	default:
-	    errs() << "Not supported architecture for " << triple.str() << "\n";
-	    return CL_EBYTECODE;
-    }
-
-    std::string cpu = sys::getHostCPUName();
-    if (cli_debug_flag)
-	errs() << "host cpu is: " << cpu << "\n";
-    if (!cpu.compare("i386") ||
-	!cpu.compare("i486")) {
-	bcs->engine = 0;
-	DEBUG(errs() << "i[34]86 detected, falling back to interpreter (JIT needs pentium or better\n");
-	/* i386 and i486 has to fallback to interpreter */
-	return 0;
-    }
-    std::string ErrMsg;
-    sys::MemoryBlock B = sys::Memory::AllocateRWX(4096, NULL, &ErrMsg);
-    if (B.base() == 0) {
-	errs() << MODULE << ErrMsg << "\n";
-#ifdef __linux__
-	errs() << MODULE << "SELinux is preventing 'execmem' access. Run 'setsebool -P clamd_use_jit on' to allow access\n";
-#endif
-	errs() << MODULE << "falling back to interpreter mode\n";
-	return 0;
-    } else {
-	sys::Memory::ReleaseRWX(B);
-    }
-
     bcs->engine = new(std::nothrow) cli_bcengine;
     if (!bcs->engine)
 	return CL_EMEM;
@@ -1937,7 +1899,7 @@ int cli_bytecode_init_jit(struct cli_all_bc *bcs, unsigned dconfmask)
     return 0;
 }
 
-int cli_bytecode_done_jit(struct cli_all_bc *bcs)
+int cli_bytecode_done_jit(struct cli_all_bc *bcs, int partial)
 {
     LLVMApiScopedLock scopedLock;
     if (bcs->engine) {
@@ -1945,10 +1907,14 @@ int cli_bytecode_done_jit(struct cli_all_bc *bcs)
 	    if (bcs->engine->Listener)
 		bcs->engine->EE->UnregisterJITEventListener(bcs->engine->Listener);
 	    delete bcs->engine->EE;
+	    bcs->engine->EE = 0;
 	}
 	delete bcs->engine->Listener;
-	delete bcs->engine;
-	bcs->engine = 0;
+	bcs->engine->Listener = 0;
+	if (!partial) {
+	    delete bcs->engine;
+	    bcs->engine = 0;
+	}
     }
     return 0;
 }
@@ -2046,6 +2012,8 @@ void cli_printcxxver()
 namespace ClamBCModule {
 void stop(const char *msg, llvm::Function* F, llvm::Instruction* I)
 {
+    if (F && F->hasName())
+	llvm::errs() << "in function " << F->getName() << ": ";
     llvm::errs() << msg << "\n";
 }
 }
