@@ -66,7 +66,11 @@ typedef enum {
     CL_EMAXSIZE,
     CL_EMAXFILES,
     CL_EFORMAT,
-    CL_EBYTECODE
+    CL_EBYTECODE,/* may be reported in testmode */
+    CL_EBYTECODE_TESTFAIL, /* may be reported in testmode */
+
+    /* no error codes below this line please */
+    CL_ELAST_ERROR
 } cl_error_t;
 
 /* db options */
@@ -109,6 +113,8 @@ typedef enum {
 #define CL_SCAN_PARTIAL_MESSAGE         0x40000
 #define CL_SCAN_HEURISTIC_PRECEDENCE    0x80000
 
+#define CL_SCAN_INTERNAL_COLLECT_SHA    0x80000000 /* Enables hash output in sha-collect builds - for internal use only */
+
 /* recommended scan settings */
 #define CL_SCAN_STDOPT		(CL_SCAN_ARCHIVE | CL_SCAN_MAIL | CL_SCAN_OLE2 | CL_SCAN_PDF | CL_SCAN_HTML | CL_SCAN_PE | CL_SCAN_ALGORITHMIC | CL_SCAN_ELF)
 
@@ -141,8 +147,9 @@ enum cl_engine_field {
     CL_ENGINE_AC_MAXDEPTH,	    /* uint32_t */
     CL_ENGINE_TMPDIR,		    /* (char *) */
     CL_ENGINE_KEEPTMP,		    /* uint32_t */
-    CL_ENGINE_BYTECODE_SECURITY,     /* uint32_t */
-    CL_ENGINE_BYTECODE_TIMEOUT       /* uint32_t */
+    CL_ENGINE_BYTECODE_SECURITY,    /* uint32_t */
+    CL_ENGINE_BYTECODE_TIMEOUT,     /* uint32_t */
+    CL_ENGINE_BYTECODE_MODE         /* uint32_t */
 };
 
 enum bytecode_security {
@@ -151,6 +158,15 @@ enum bytecode_security {
     CL_BYTECODE_TRUST_NOTHING /* paranoid setting */
 };
 
+enum bytecode_mode {
+    CL_BYTECODE_MODE_AUTO=0, /* JIT if possible, fallback to interpreter */
+    CL_BYTECODE_MODE_JIT, /* force JIT */
+    CL_BYTECODE_MODE_INTERPRETER, /* force interpreter */
+    CL_BYTECODE_MODE_TEST, /* both JIT and interpreter, compare results,
+			      all failures are fatal */
+    CL_BYTECODE_MODE_OFF /* for query only, not settable */
+};
+
 extern int cl_engine_set_num(struct cl_engine *engine, enum cl_engine_field field, long long num);
 
 extern long long cl_engine_get_num(const struct cl_engine *engine, enum cl_engine_field field, int *err);
@@ -172,6 +188,57 @@ extern int cl_engine_addref(struct cl_engine *engine);
 extern int cl_engine_free(struct cl_engine *engine);
 
 
+/* CALLBACKS - WARNING: unstable API - WIP */
+
+
+typedef cl_error_t (*clcb_pre_scan)(int fd, void *context);
+/* PRE-SCAN
+Input:
+fd      = File descriptor which is about to be scanned
+context = Opaque application provided data
+
+Output:
+CL_CLEAN = File is scanned
+CL_BREAK = Whitelisted by callback - file is skipped and marked as clean
+CL_VIRUS = Blacklisted by callback - file is skipped and marked as infected
+*/
+extern void cl_engine_set_clcb_pre_scan(struct cl_engine *engine, clcb_pre_scan callback);
+
+
+typedef cl_error_t (*clcb_post_scan)(int fd, int result, const char *virname, void *context);
+/* POST-SCAN
+Input:
+fd      = File descriptor which is was scanned
+result  = The scan result for the file
+virname = Virus name if infected
+context = Opaque application provided data
+
+Output:
+CL_CLEAN = Scan result is not overridden
+CL_BREAK = Whitelisted by callback - scan result is set to CL_CLEAN
+CL_VIRUS = Blacklisted by callback - scan result is set to CL_VIRUS
+*/
+extern void cl_engine_set_clcb_post_scan(struct cl_engine *engine, clcb_post_scan callback);
+
+
+typedef int (*clcb_sigload)(const char *type, const char *name, void *context);
+/* SIGNATURE LOAD
+Input:
+type = The signature type (e.g. "db", "ndb", "mdb", etc.)
+name = The virus name
+context = Opaque application provided data
+
+Output:
+0     = Load the current signature
+Non 0 = Skip the current signature
+
+WARNING: Some signatures (notably ldb, cbc) can be dependent upon other signatures.
+         Failure to preserve dependency chains will result in database loading failure.
+         It is the implementor's responsibility to guarantee consistency.
+*/
+extern void cl_engine_set_clcb_sigload(struct cl_engine *engine, clcb_sigload callback, void *context);
+
+
 struct cl_stat {
     char *dir;
     struct stat *stattab;
@@ -193,6 +260,7 @@ struct cl_cvd {		    /* field no. */
 
 /* file scanning */
 extern int cl_scandesc(int desc, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, unsigned int scanoptions);
+extern int cl_scandesc_callback(int desc, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, unsigned int scanoptions, void *context);
 
 extern int cl_scanfile(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, unsigned int scanoptions);
 
