@@ -34,6 +34,7 @@
 #include "others.h"
 #include "matcher-ac.h"
 #include "matcher-bm.h"
+#include "matcher-md5.h"
 #include "md5.h"
 #include "filetypes.h"
 #include "matcher.h"
@@ -41,6 +42,7 @@
 #include "elf.h"
 #include "execs.h"
 #include "special.h"
+#include "scanners.h"
 #include "str.h"
 #include "cltypes.h"
 #include "default.h"
@@ -50,6 +52,8 @@
 #include "regex/regex.h"
 #include "filtering.h"
 #include "perflogging.h"
+#include "bytecode_priv.h"
+#include "bytecode_api_impl.h"
 
 #ifdef HAVE__INTERNAL__SHA_COLLECT
 #include "sha256.h"
@@ -83,9 +87,11 @@ static inline int matcher_run(const struct cli_matcher *root,
 			      const unsigned char *buffer, uint32_t length,
 			      const char **virname, struct cli_ac_data *mdata,
 			      uint32_t offset,
+			      const struct cli_target_info *tinfo,
 			      cli_file_t ftype,
 			      struct cli_matched_type **ftoffset,
 			      unsigned int acmode,
+			      struct cli_ac_result **acres,
 			      fmap_t *map,
 			      struct cli_bm_off *offdata)
 {
@@ -124,15 +130,15 @@ static inline int matcher_run(const struct cli_matcher *root,
 	    /* Don't use prefiltering for BM offset mode, since BM keeps tracks
 	     * of offsets itself, and doesn't work if we skip chunks of input
 	     * data */
-	    ret = cli_bm_scanbuff(orig_buffer, orig_length, virname, NULL, root, orig_offset, map, offdata);
+	    ret = cli_bm_scanbuff(orig_buffer, orig_length, virname, NULL, root, orig_offset, tinfo, offdata);
 	} else {
-	    ret = cli_bm_scanbuff(buffer, length, virname, NULL, root, offset, map, offdata);
+	    ret = cli_bm_scanbuff(buffer, length, virname, NULL, root, offset, tinfo, offdata);
 	}
 	if (ret == CL_VIRUS)
 	    return ret;
     }
     PERF_LOG_TRIES(acmode, 0, length);
-    ret = cli_ac_scanbuff(buffer, length, virname, NULL, NULL, root, mdata, offset, ftype, ftoffset, acmode, NULL);
+    ret = cli_ac_scanbuff(buffer, length, virname, NULL, acres, root, mdata, offset, ftype, ftoffset, acmode, NULL);
     return ret;
 }
 
@@ -166,19 +172,19 @@ int cli_scanbuff(const unsigned char *buffer, uint32_t length, uint32_t offset,
 	if(!acdata && (ret = cli_ac_initdata(&mdata, troot->ac_partsigs, troot->ac_lsigs, troot->ac_reloff_num, CLI_DEFAULT_AC_TRACKLEN)))
 	    return ret;
 
-	ret = matcher_run(troot, buffer, length, virname, acdata ? (acdata[0]): (&mdata), offset, ftype, NULL, AC_SCAN_VIR, *ctx->fmap, NULL);
+	ret = matcher_run(troot, buffer, length, virname, acdata ? (acdata[0]): (&mdata), offset, NULL, ftype, NULL, AC_SCAN_VIR, NULL, *ctx->fmap, NULL);
 
 	if(!acdata)
 	    cli_ac_freedata(&mdata);
 
-	if(ret == CL_VIRUS)
+	if(ret == CL_VIRUS || ret == CL_EMEM)
 	    return ret;
     }
 
     if(!acdata && (ret = cli_ac_initdata(&mdata, groot->ac_partsigs, groot->ac_lsigs, groot->ac_reloff_num, CLI_DEFAULT_AC_TRACKLEN)))
 	return ret;
 
-    ret = matcher_run(groot, buffer, length, virname, acdata ? (acdata[1]): (&mdata), offset, ftype, NULL, AC_SCAN_VIR, *ctx->fmap, NULL);
+    ret = matcher_run(groot, buffer, length, virname, acdata ? (acdata[1]): (&mdata), offset, NULL, ftype, NULL, AC_SCAN_VIR, NULL, *ctx->fmap, NULL);
 
     if(!acdata)
 	cli_ac_freedata(&mdata);
@@ -192,9 +198,8 @@ int cli_scanbuff(const unsigned char *buffer, uint32_t length, uint32_t offset,
  * offdata[2]: max shift
  * offdata[3]: section number
  */
-int cli_caloff(const char *offstr, struct cli_target_info *info, fmap_t *map, unsigned int target, uint32_t *offdata, uint32_t *offset_min, uint32_t *offset_max)
+int cli_caloff(const char *offstr, const struct cli_target_info *info, unsigned int target, uint32_t *offdata, uint32_t *offset_min, uint32_t *offset_max)
 {
-	int (*einfo)(fmap_t *, struct cli_exe_info *) = NULL;
 	char offcpy[65];
 	unsigned int n, val;
 	char *pt;
@@ -307,33 +312,6 @@ int cli_caloff(const char *offstr, struct cli_target_info *info, fmap_t *map, un
 	    return CL_SUCCESS;
 	}
 
-	if((offdata[0] == CLI_OFF_EOF_MINUS)) {
-	    if(!info->fsize)
-		info->fsize = map->len;
-	} else if(!info->status) {
-	    if(target == 1)
-		einfo = cli_peheader;
-	    else if(target == 6)
-		einfo = cli_elfheader;
-	    else if(target == 9)
-		einfo = cli_machoheader;
-
-	    if(!einfo) {
-		cli_errmsg("cli_caloff: Invalid offset/filetype\n");
-		return CL_EMALFDB;
-	    }
-
-	    if(einfo(map, &info->exeinfo)) {
-		/* einfo *may* fail */
-		info->status = -1;
-		*offset_min = CLI_OFF_NONE;
-		if(offset_max)
-		    *offset_max = CLI_OFF_NONE;
-		return CL_SUCCESS;
-	    }
-	    info->status = 1;
-	}
-
 	switch(offdata[0]) {
 	    case CLI_OFF_EOF_MINUS:
 		*offset_min = info->fsize - offdata[1];
@@ -376,15 +354,36 @@ int cli_caloff(const char *offstr, struct cli_target_info *info, fmap_t *map, un
     return CL_SUCCESS;
 }
 
+static void targetinfo(struct cli_target_info *info, unsigned int target, fmap_t *map)
+{
+	int (*einfo)(fmap_t *, struct cli_exe_info *) = NULL;
+
+
+    memset(info, 0, sizeof(struct cli_target_info));
+    info->fsize = map->len;
+    cli_hashset_init_noalloc(&info->exeinfo.vinfo);
+
+    if(target == 1)
+	einfo = cli_peheader;
+    else if(target == 6)
+	einfo = cli_elfheader;
+    else if(target == 9)
+	einfo = cli_machoheader;
+    else return;
+
+    if(einfo(map, &info->exeinfo))
+	info->status = -1;
+    else
+	info->status = 1;
+}
+
 int cli_checkfp(unsigned char *digest, size_t size, cli_ctx *ctx)
 {
 	char md5[33];
 	unsigned int i;
 	const char *virname;
-	const struct cli_bm_patt *patt = NULL;
 
-
-    if(ctx->engine->md5_fp && cli_bm_scanbuff(digest, 16, &virname, &patt, ctx->engine->md5_fp, 0, NULL, NULL) == CL_VIRUS && patt->filesize == size) {
+    if(ctx->engine->md5_fp && cli_md5m_scan(digest, size, &virname, ctx->engine->md5_fp) == CL_VIRUS) {
 	cli_dbgmsg("cli_checkfp(): Found false positive detection (fp sig: %s)\n", virname);
 	return CL_CLEAN;
     }
@@ -422,26 +421,82 @@ int cli_checkfp(unsigned char *digest, size_t size, cli_ctx *ctx)
     }
 #endif
 
+    if(ctx->virsize && !*ctx->virsize) {
+	if(ctx->virsize)
+	    *ctx->virsize = size;
+	if(ctx->virhash)
+	    strcpy(ctx->virhash, md5);
+    }
+
     return CL_VIRUS;
 }
 
-static int matchicon(cli_ctx *ctx, const char *grp1, const char *grp2)
+static int matchicon(cli_ctx *ctx, struct cli_exe_info *exeinfo, const char *grp1, const char *grp2)
 {
 	icon_groupset iconset;
 
+    if(!ctx ||
+       !ctx->engine ||
+       !ctx->engine->iconcheck ||
+       !ctx->engine->iconcheck->group_counts[0] ||
+       !ctx->engine->iconcheck->group_counts[1] ||
+       !exeinfo->res_addr
+    ) return CL_CLEAN;
+
     cli_icongroupset_init(&iconset);
     cli_icongroupset_add(grp1 ? grp1 : "*", &iconset, 0, ctx);
     cli_icongroupset_add(grp2 ? grp2 : "*", &iconset, 1, ctx);
-    return cli_match_icon(&iconset, ctx);
+    return cli_scanicon(&iconset, exeinfo->res_addr, ctx, exeinfo->section, exeinfo->nsections, exeinfo->hdr_size);
 }
 
-int cli_scandesc(int desc, cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli_matched_type **ftoffset, unsigned int acmode)
+int32_t cli_bcapi_matchicon(struct cli_bc_ctx *ctx , const uint8_t* grp1, int32_t grp1len,
+			    const uint8_t* grp2, int32_t grp2len)
+{
+    int ret;
+    char group1[128], group2[128];
+    const char **oldvirname;
+    struct cli_exe_info info;
+
+    if (!ctx->hooks.pedata->ep) {
+	cli_dbgmsg("bytecode: matchicon only works with PE files\n");
+	return -1;
+    }
+    if ((size_t) grp1len > sizeof(group1)-1 ||
+	(size_t) grp2len > sizeof(group2)-1)
+	return -1;
+    oldvirname = ((cli_ctx*)ctx->ctx)->virname;
+    ((cli_ctx*)ctx->ctx)->virname = NULL;
+    memcpy(group1, grp1, grp1len);
+    memcpy(group2, grp2, grp2len);
+    group1[grp1len] = 0;
+    group2[grp2len] = 0;
+    memset(&info, 0, sizeof(info));
+    if (ctx->bc->kind == BC_PE_UNPACKER || ctx->bc->kind == BC_PE_ALL) {
+	if(le16_to_host(ctx->hooks.pedata->file_hdr.Characteristics) & 0x2000 ||
+	   !ctx->hooks.pedata->dirs[2].Size)
+	    info.res_addr = 0;
+	else
+	    info.res_addr = le32_to_host(ctx->hooks.pedata->dirs[2].VirtualAddress);
+    } else
+	info.res_addr = ctx->resaddr; /* from target_info */
+    info.section = (struct cli_exe_section*)ctx->sections;
+    info.nsections = ctx->hooks.pedata->nsections;
+    info.hdr_size = ctx->hooks.pedata->hdr_size;
+    cli_dbgmsg("bytecode matchicon %s %s\n", group1, group2);
+    ret = matchicon(ctx->ctx, &info, group1[0] ? group1 : NULL,
+		    group2[0] ? group2 : NULL);
+    ((cli_ctx*)ctx->ctx)->virname = oldvirname;
+    return ret;
+}
+
+
+int cli_scandesc(int desc, cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli_matched_type **ftoffset, unsigned int acmode, struct cli_ac_result **acres)
 {
     int ret = CL_EMEM, empty;
     fmap_t *map = *ctx->fmap;
 
     if((*ctx->fmap = fmap_check_empty(desc, 0, 0, &empty))) {
-	ret = cli_fmap_scandesc(ctx, ftype, ftonly, ftoffset, acmode, NULL);
+	ret = cli_fmap_scandesc(ctx, ftype, ftonly, ftoffset, acmode, acres, NULL);
 	map->dont_cache_flag = (*ctx->fmap)->dont_cache_flag;
 	funmap(*ctx->fmap);
     }
@@ -451,12 +506,10 @@ int cli_scandesc(int desc, cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struc
     return ret;
 }
 
-int cli_lsig_eval(cli_ctx *ctx, struct cli_matcher *root, struct cli_ac_data *acdata)
+int cli_lsig_eval(cli_ctx *ctx, struct cli_matcher *root, struct cli_ac_data *acdata, struct cli_target_info *target_info)
 {
 	unsigned int i, evalcnt;
 	uint64_t evalids;
-	int (*einfo)(fmap_t *, struct cli_exe_info *) = NULL;
-	struct cli_exe_info exeinfo;
 	fmap_t *map = *ctx->fmap;
 
     for(i = 0; i < root->ac_lsigs; i++) {
@@ -470,40 +523,40 @@ int cli_lsig_eval(cli_ctx *ctx, struct cli_matcher *root, struct cli_ac_data *ac
 		continue;
 
 	    if(root->ac_lsigtable[i]->tdb.ep || root->ac_lsigtable[i]->tdb.nos) {
-		einfo = NULL;
-		if(root->type == 1)
-		    einfo = cli_peheader;
-		else if(root->type == 6)
-		    einfo = cli_elfheader;
-		else if(root->type == 9)
-		    einfo = cli_machoheader;
-		if(!einfo)
+		if(!target_info || target_info->status != 1)
 		    continue;
-		memset(&exeinfo, 0, sizeof(exeinfo));
-		if(einfo(map, &exeinfo))
+		if(root->ac_lsigtable[i]->tdb.ep && (root->ac_lsigtable[i]->tdb.ep[0] > target_info->exeinfo.ep || root->ac_lsigtable[i]->tdb.ep[1] < target_info->exeinfo.ep))
 		    continue;
-		if(exeinfo.section)
-		    free(exeinfo.section);
-		if(root->ac_lsigtable[i]->tdb.ep && (root->ac_lsigtable[i]->tdb.ep[0] > exeinfo.ep || root->ac_lsigtable[i]->tdb.ep[1] < exeinfo.ep))
-		    continue;
-		if(root->ac_lsigtable[i]->tdb.nos && (root->ac_lsigtable[i]->tdb.nos[0] > exeinfo.nsections || root->ac_lsigtable[i]->tdb.nos[1] < exeinfo.nsections))
+		if(root->ac_lsigtable[i]->tdb.nos && (root->ac_lsigtable[i]->tdb.nos[0] > target_info->exeinfo.nsections || root->ac_lsigtable[i]->tdb.nos[1] < target_info->exeinfo.nsections))
 		    continue;
 	    }
-	    if(root->ac_lsigtable[i]->tdb.icongrp1 || root->ac_lsigtable[i]->tdb.icongrp2) {
-		if(matchicon(ctx, root->ac_lsigtable[i]->tdb.icongrp1, root->ac_lsigtable[i]->tdb.icongrp2) == CL_VIRUS) {
-		    if(ctx->virname)
-			*ctx->virname = root->ac_lsigtable[i]->virname;
+
+	    if(root->ac_lsigtable[i]->tdb.handlertype) {
+		if(cli_magic_scandesc_type(map->fd, ctx, root->ac_lsigtable[i]->tdb.handlertype[0]) == CL_VIRUS)
 		    return CL_VIRUS;
-		} else {
+		continue;
+	    }
+
+	    if(root->ac_lsigtable[i]->tdb.icongrp1 || root->ac_lsigtable[i]->tdb.icongrp2) {
+		if(!target_info || target_info->status != 1)
 		    continue;
+		if(matchicon(ctx, &target_info->exeinfo, root->ac_lsigtable[i]->tdb.icongrp1, root->ac_lsigtable[i]->tdb.icongrp2) == CL_VIRUS) {
+		    if(!root->ac_lsigtable[i]->bc_idx) {
+			if(ctx->virname)
+			    *ctx->virname = root->ac_lsigtable[i]->virname;
+			return CL_VIRUS;
+		    } else if(cli_bytecode_runlsig(ctx, target_info, &ctx->engine->bcs, root->ac_lsigtable[i]->bc_idx, ctx->virname, acdata->lsigcnt[i], acdata->lsigsuboff[i], map) == CL_VIRUS) {
+			return CL_VIRUS;
+		    }
 		}
+		continue;
 	    }
 	    if(!root->ac_lsigtable[i]->bc_idx) {
 		if(ctx->virname)
 		    *ctx->virname = root->ac_lsigtable[i]->virname;
 		return CL_VIRUS;
 	    }
-	    if(cli_bytecode_runlsig(ctx, &ctx->engine->bcs, root->ac_lsigtable[i]->bc_idx, ctx->virname, acdata->lsigcnt[i], acdata->lsigsuboff[i], map) == CL_VIRUS) {
+	    if(cli_bytecode_runlsig(ctx, target_info, &ctx->engine->bcs, root->ac_lsigtable[i]->bc_idx, ctx->virname, acdata->lsigcnt[i], acdata->lsigsuboff[i], map) == CL_VIRUS) {
 		return CL_VIRUS;
 	    }
 	}
@@ -511,17 +564,18 @@ int cli_lsig_eval(cli_ctx *ctx, struct cli_matcher *root, struct cli_ac_data *ac
     return CL_CLEAN;
 }
 
-int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli_matched_type **ftoffset, unsigned int acmode, unsigned char *refhash)
+int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli_matched_type **ftoffset, unsigned int acmode, struct cli_ac_result **acres, unsigned char *refhash)
 {
  	unsigned char *buff;
 	int ret = CL_CLEAN, type = CL_CLEAN, bytes;
-	unsigned int i, bm_offmode = 0;
+	unsigned int i = 0, bm_offmode = 0;
 	uint32_t maxpatlen, offset = 0;
 	struct cli_ac_data gdata, tdata;
 	struct cli_bm_off toff;
 	cli_md5_ctx md5ctx;
 	unsigned char digest[16];
 	struct cli_matcher *groot = NULL, *troot = NULL;
+	struct cli_target_info info;
 	fmap_t *map = *ctx->fmap;
 
     if(!ctx->engine) {
@@ -553,22 +607,34 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
 	    maxpatlen = groot->maxpatlen;
     }
 
+    targetinfo(&info, i, map);
+
     if(!ftonly)
-	if((ret = cli_ac_initdata(&gdata, groot->ac_partsigs, groot->ac_lsigs, groot->ac_reloff_num, CLI_DEFAULT_AC_TRACKLEN)) || (ret = cli_ac_caloff(groot, &gdata, map)))
+	if((ret = cli_ac_initdata(&gdata, groot->ac_partsigs, groot->ac_lsigs, groot->ac_reloff_num, CLI_DEFAULT_AC_TRACKLEN)) || (ret = cli_ac_caloff(groot, &gdata, &info))) {
+	    if(info.exeinfo.section)
+		free(info.exeinfo.section);
+	    cli_hashset_destroy(&info.exeinfo.vinfo);
 	    return ret;
+	}
 
     if(troot) {
-	if((ret = cli_ac_initdata(&tdata, troot->ac_partsigs, troot->ac_lsigs, troot->ac_reloff_num, CLI_DEFAULT_AC_TRACKLEN)) || (ret = cli_ac_caloff(troot, &tdata, map))) {
+	if((ret = cli_ac_initdata(&tdata, troot->ac_partsigs, troot->ac_lsigs, troot->ac_reloff_num, CLI_DEFAULT_AC_TRACKLEN)) || (ret = cli_ac_caloff(troot, &tdata, &info))) {
 	    if(!ftonly)
 		cli_ac_freedata(&gdata);
+	    if(info.exeinfo.section)
+		free(info.exeinfo.section);
+	    cli_hashset_destroy(&info.exeinfo.vinfo);
 	    return ret;
 	}
 	if(troot->bm_offmode) {
 	    if(map->len >= CLI_DEFAULT_BM_OFFMODE_FSIZE) {
-		if((ret = cli_bm_initoff(troot, &toff, map))) {
+		if((ret = cli_bm_initoff(troot, &toff, &info))) {
 		    if(!ftonly)
 			cli_ac_freedata(&gdata);
 		    cli_ac_freedata(&tdata);
+		    if(info.exeinfo.section)
+			free(info.exeinfo.section);
+		    cli_hashset_destroy(&info.exeinfo.vinfo);
 		    return ret;
 		}
 		bm_offmode = 1;
@@ -587,28 +653,35 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
 	    *ctx->scanned += bytes / CL_COUNT_PRECISION;
 
 	if(troot) {
-	    ret = matcher_run(troot, buff, bytes, ctx->virname, &tdata, offset, ftype, ftoffset, acmode, map, bm_offmode ? &toff : NULL);
+	    ret = matcher_run(troot, buff, bytes, ctx->virname, &tdata, offset, &info, ftype, ftoffset, acmode, acres, map, bm_offmode ? &toff : NULL);
 
-	    if(ret == CL_VIRUS) {
+	    if(ret == CL_VIRUS || ret == CL_EMEM) {
 		if(!ftonly)
 		    cli_ac_freedata(&gdata);
 		cli_ac_freedata(&tdata);
 		if(bm_offmode)
 		    cli_bm_freeoff(&toff);
-		return CL_VIRUS;
+		if(info.exeinfo.section)
+		    free(info.exeinfo.section);
+		cli_hashset_destroy(&info.exeinfo.vinfo);
+		return ret;
 	    }
 	}
 
 	if(!ftonly) {
-	    ret = matcher_run(groot, buff, bytes, ctx->virname, &gdata, offset, ftype, ftoffset, acmode, map, NULL);
-	    if(ret == CL_VIRUS) {
+	    ret = matcher_run(groot, buff, bytes, ctx->virname, &gdata, offset, &info, ftype, ftoffset, acmode, acres, map, NULL);
+
+	    if(ret == CL_VIRUS || ret == CL_EMEM) {
 		cli_ac_freedata(&gdata);
 		if(troot) {
 		    cli_ac_freedata(&tdata);
 		    if(bm_offmode)
 			cli_bm_freeoff(&toff);
 		}
-		return CL_VIRUS;
+		if(info.exeinfo.section)
+		    free(info.exeinfo.section);
+		cli_hashset_destroy(&info.exeinfo.vinfo);
+		return ret;
 	    } else if((acmode & AC_SCAN_FT) && ret >= CL_TYPENO) {
 		if(ret > type)
 		    type = ret;
@@ -623,7 +696,7 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
     }
 
     if(troot) {
-	ret = cli_lsig_eval(ctx, troot, &tdata);
+	ret = cli_lsig_eval(ctx, troot, &tdata, &info);
 	cli_ac_freedata(&tdata);
 	if(bm_offmode)
 	    cli_bm_freeoff(&toff);
@@ -631,20 +704,23 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
 
     if(groot) {
 	if(ret != CL_VIRUS)
-	    ret = cli_lsig_eval(ctx, groot, &gdata);
+	    ret = cli_lsig_eval(ctx, groot, &gdata, &info);
 	cli_ac_freedata(&gdata);
     }
 
+    if(info.exeinfo.section)
+	free(info.exeinfo.section);
+    cli_hashset_destroy(&info.exeinfo.vinfo);
+
     if(ret == CL_VIRUS)
 	return CL_VIRUS;
 
     if(!ftonly && ctx->engine->md5_hdb) {
-	    const struct cli_bm_patt *patt;
 	if(!refhash) {
 	    cli_md5_final(digest, &md5ctx);
 	    refhash = digest;
 	}
-	if(cli_bm_scanbuff(refhash, 16, ctx->virname, &patt, ctx->engine->md5_hdb, 0, NULL, NULL) == CL_VIRUS && patt->filesize == map->len && (cli_bm_scanbuff(refhash, 16, NULL, &patt, ctx->engine->md5_fp, 0, NULL, NULL) != CL_VIRUS || patt->filesize != map->len))
+	if(cli_md5m_scan(refhash, map->len, ctx->virname, ctx->engine->md5_hdb) == CL_VIRUS && cli_md5m_scan(refhash, map->len, NULL, ctx->engine->md5_fp) != CL_VIRUS)
 	    return CL_VIRUS;
     }
 
