@@ -53,7 +53,7 @@
  * in re-enabling affected modules.
  */
 
-#define CL_FLEVEL 53
+#define CL_FLEVEL 55
 #define CL_FLEVEL_DCONF	CL_FLEVEL
 #define CL_FLEVEL_SIGTOOL CL_FLEVEL
 
@@ -76,7 +76,7 @@ extern uint8_t cli_debug_flag;
   ((bb_size) > 0 && (sb_size) >= 0 && (size_t)(sb_size) <= (size_t)(bb_size) \
    && (sb) >= (bb) && ((sb) + (sb_size)) <= ((bb) + (bb_size)) && ((sb) + (sb_size)) >= (bb) && (sb) < ((bb) + (bb_size)))
 
-#define CLI_MAX_ALLOCATION 184549376
+#define CLI_MAX_ALLOCATION (182*1024*1024)
 
 #ifdef	HAVE_SYS_PARAM_H
 #include <sys/param.h>	/* for NAME_MAX */
@@ -109,6 +109,8 @@ typedef struct bitset_tag
 /* internal clamav context */
 typedef struct cli_ctx_tag {
     const char **virname;
+    char *virhash;
+    unsigned int *virsize;
     unsigned long int *scanned;
     const struct cli_matcher *root;
     const struct cl_engine *engine;
@@ -117,11 +119,13 @@ typedef struct cli_ctx_tag {
     unsigned int recursion;
     unsigned int scannedfiles;
     unsigned int found_possibly_unwanted;
+    unsigned int corrupted_input;
     cli_file_t container_type; /* FIXME: to be made into a stack or array - see bb#1579 & bb#1293 */
     size_t container_size;
     struct cli_dconf *dconf;
     fmap_t **fmap;
     bitset_t* hook_lsig_matches;
+    void *cb_ctx;
 #ifdef HAVE__INTERNAL__SHA_COLLECT
     char entry_filename[2048];
     int sha_collect;
@@ -246,6 +250,12 @@ struct cl_engine {
     /* Used for memory pools */
     mpool_t *mempool;
 
+    /* Callback(s) */
+    clcb_pre_scan cb_pre_scan;
+    clcb_post_scan cb_post_scan;
+    clcb_sigload cb_sigload;
+    void *cb_sigload_ctx;
+
     /* Used for bytecode */
     struct cli_all_bc bcs;
     unsigned *hooks[_BC_LAST_HOOK - _BC_START_HOOKS];
@@ -253,6 +263,7 @@ struct cl_engine {
     unsigned hook_lsig_ids;
     enum bytecode_security bytecode_security;
     uint32_t bytecode_timeout;
+    enum bytecode_mode bytecode_mode;
 };
 
 struct cl_settings {
@@ -331,6 +342,10 @@ union unaligned_16 {
 	int16_t una_s16;
 } __attribute__((packed));
 
+struct unaligned_ptr {
+    void *ptr;
+} __attribute__((packed));
+
 #ifdef HAVE_PRAGMA_PACK
 #pragma pack()
 #endif
