@@ -54,11 +54,10 @@
 #include "mew.h"
 #include "upack.h"
 #include "matcher.h"
-#include "matcher-bm.h"
+#include "matcher-md5.h"
 #include "disasm.h"
 #include "special.h"
 #include "ishield.h"
-#include "pe_icons.h"
 
 #define DCONF ctx->dconf->pe
 
@@ -190,6 +189,8 @@ FSGSTUFF; \
 #define CLI_UNPRESULTSFSG1(NAME,EXPR,GOOD,FREEME) CLI_UNPRESULTS_(NAME,FSGCASE(NAME,free(sections)),EXPR,GOOD,FREEME)
 #define CLI_UNPRESULTSFSG2(NAME,EXPR,GOOD,FREEME) CLI_UNPRESULTS_(NAME,FSGCASE(NAME,(void)0),EXPR,GOOD,FREEME)
 
+#define DETECT_BROKEN_PE (DETECT_BROKEN && !ctx->corrupted_input)
+
 struct offset_list {
     uint32_t offset;
     struct offset_list *next;
@@ -499,7 +500,7 @@ static void cli_parseres_special(uint32_t base, uint32_t rva, fmap_t *map, struc
     fmap_unneed_ptr(map, oentry, entries*8);
 }
 
-int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
+int cli_scanpe(cli_ctx *ctx)
 {
 	uint16_t e_magic; /* DOS signature ("MZ") */
 	uint16_t nsections;
@@ -552,7 +553,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
     if(fmap_readn(map, &e_lfanew, 58 + sizeof(e_magic), sizeof(e_lfanew)) != sizeof(e_lfanew)) {
 	cli_dbgmsg("Can't read new header address\n");
 	/* truncated header? */
-	if(DETECT_BROKEN) {
+	if(DETECT_BROKEN_PE) {
 	    if(ctx->virname)
 		*ctx->virname = "Heuristics.Broken.Executable";
 	    return CL_VIRUS;
@@ -682,15 +683,17 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 
     nsections = EC16(file_hdr.NumberOfSections);
     if(nsections < 1 || nsections > 96) {
-	if(DETECT_BROKEN) {
+	if(DETECT_BROKEN_PE) {
 	    if(ctx->virname)
 		*ctx->virname = "Heuristics.Broken.Executable";
 	    return CL_VIRUS;
 	}
-	if(nsections)
-	    cli_warnmsg("PE file contains %d sections\n", nsections);
-	else
-	    cli_warnmsg("PE file contains no sections\n");
+	if(!ctx->corrupted_input) {
+	    if(nsections)
+		cli_warnmsg("PE file contains %d sections\n", nsections);
+	    else
+		cli_warnmsg("PE file contains no sections\n");
+	}
 	return CL_CLEAN;
     }
     cli_dbgmsg("NumberOfSections: %d\n", nsections);
@@ -702,7 +705,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 
     if (EC16(file_hdr.SizeOfOptionalHeader) < sizeof(struct pe_image_optional_hdr32)) {
         cli_dbgmsg("SizeOfOptionalHeader too small\n");
-	if(DETECT_BROKEN) {
+	if(DETECT_BROKEN_PE) {
 	    if(ctx->virname)
 	        *ctx->virname = "Heuristics.Broken.Executable";
 	    return CL_VIRUS;
@@ -713,7 +716,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
     at = e_lfanew + sizeof(struct pe_image_file_hdr);
     if(fmap_readn(map, &optional_hdr32, at, sizeof(struct pe_image_optional_hdr32)) != sizeof(struct pe_image_optional_hdr32)) {
         cli_dbgmsg("Can't read optional file header\n");
-	if(DETECT_BROKEN) {
+	if(DETECT_BROKEN_PE) {
 	    if(ctx->virname)
 	        *ctx->virname = "Heuristics.Broken.Executable";
 	    return CL_VIRUS;
@@ -727,7 +730,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
         if(EC16(file_hdr.SizeOfOptionalHeader)!=sizeof(struct pe_image_optional_hdr64)) {
 	    /* FIXME: need to play around a bit more with xp64 */
 	    cli_dbgmsg("Incorrect SizeOfOptionalHeader for PE32+\n");
-	    if(DETECT_BROKEN) {
+	    if(DETECT_BROKEN_PE) {
 	        if(ctx->virname)
 		    *ctx->virname = "Heuristics.Broken.Executable";
 		return CL_VIRUS;
@@ -742,8 +745,9 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 	    either way it's a 32bit thingy
 	*/
         if(EC16(optional_hdr32.Magic) != PE32_SIGNATURE) {
-	    cli_warnmsg("Incorrect magic number in optional header\n");
-	    if(DETECT_BROKEN) {
+	    if(!ctx->corrupted_input)
+		cli_warnmsg("Incorrect magic number in optional header\n");
+	    if(DETECT_BROKEN_PE) {
 	        if(ctx->virname)
 		    *ctx->virname = "Heuristics.Broken.Executable";
 		return CL_VIRUS;
@@ -785,7 +789,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
         /* read the remaining part of the header */
         if(fmap_readn(map, &optional_hdr32 + 1, at, sizeof(struct pe_image_optional_hdr64) - sizeof(struct pe_image_optional_hdr32)) != sizeof(struct pe_image_optional_hdr64) - sizeof(struct pe_image_optional_hdr32)) {
 	    cli_dbgmsg("Can't read optional file header\n");
-	    if(DETECT_BROKEN) {
+	    if(DETECT_BROKEN_PE) {
 	        if(ctx->virname)
 		    *ctx->virname = "Heuristics.Broken.Executable";
 		return CL_VIRUS;
@@ -865,14 +869,14 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 
     cli_dbgmsg("------------------------------------\n");
 
-    if (DETECT_BROKEN && !native && (!(pe_plus?EC32(optional_hdr64.SectionAlignment):EC32(optional_hdr32.SectionAlignment)) || (pe_plus?EC32(optional_hdr64.SectionAlignment):EC32(optional_hdr32.SectionAlignment))%0x1000)) {
+    if (DETECT_BROKEN_PE && !native && (!(pe_plus?EC32(optional_hdr64.SectionAlignment):EC32(optional_hdr32.SectionAlignment)) || (pe_plus?EC32(optional_hdr64.SectionAlignment):EC32(optional_hdr32.SectionAlignment))%0x1000)) {
         cli_dbgmsg("Bad virtual alignemnt\n");
         if(ctx->virname)
 	    *ctx->virname = "Heuristics.Broken.Executable";
 	return CL_VIRUS;
     }
 
-    if (DETECT_BROKEN && !native && (!(pe_plus?EC32(optional_hdr64.FileAlignment):EC32(optional_hdr32.FileAlignment)) || (pe_plus?EC32(optional_hdr64.FileAlignment):EC32(optional_hdr32.FileAlignment))%0x200)) {
+    if (DETECT_BROKEN_PE && !native && (!(pe_plus?EC32(optional_hdr64.FileAlignment):EC32(optional_hdr32.FileAlignment)) || (pe_plus?EC32(optional_hdr64.FileAlignment):EC32(optional_hdr32.FileAlignment))%0x200)) {
         cli_dbgmsg("Bad file alignemnt\n");
 	if(ctx->virname)
 	    *ctx->virname = "Heuristics.Broken.Executable";
@@ -904,7 +908,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 	cli_dbgmsg("Possibly broken PE file\n");
 	free(section_hdr);
 	free(exe_sections);
-	if(DETECT_BROKEN) {
+	if(DETECT_BROKEN_PE) {
 	    if(ctx->virname)
 		*ctx->virname = "Heuristics.Broken.Executable";
 	    return CL_VIRUS;
@@ -971,7 +975,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 
 	cli_dbgmsg("------------------------------------\n");
 
-	if (DETECT_BROKEN && (!valign || (exe_sections[i].urva % valign))) { /* Bad virtual alignment */
+	if (DETECT_BROKEN_PE && (!valign || (exe_sections[i].urva % valign))) { /* Bad virtual alignment */
 	    cli_dbgmsg("VirtualAddress is misaligned\n");
 	    if(ctx->virname)
 	        *ctx->virname = "Heuristics.Broken.Executable";
@@ -985,7 +989,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 	      cli_dbgmsg("Broken PE file - Section %d starts beyond the end of file (Offset@ %lu, Total filesize %lu)\n", i, (unsigned long)exe_sections[i].raw, (unsigned long)fsize);
 		free(section_hdr);
 		free(exe_sections);
-		if(DETECT_BROKEN) {
+		if(DETECT_BROKEN_PE) {
 		    if(ctx->virname)
 		        *ctx->virname = "Heuristics.Broken.Executable";
 		    return CL_VIRUS;
@@ -1002,9 +1006,9 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 		for(j = 0; j < md5_sect->soff_len && md5_sect->soff[j] <= exe_sections[i].rsz; j++) {
 		    if(md5_sect->soff[j] == exe_sections[i].rsz) {
 			unsigned char md5_dig[16];
-			const struct cli_bm_patt *patt;
-			if(cli_md5sect(map, &exe_sections[i], md5_dig) && cli_bm_scanbuff(md5_dig, 16, ctx->virname, &patt, ctx->engine->md5_mdb, 0, NULL, NULL) == CL_VIRUS && patt->filesize == exe_sections[i].rsz) {
-			    if(cli_bm_scanbuff(md5_dig, 16, NULL, &patt, ctx->engine->md5_fp, 0, NULL, NULL) != CL_VIRUS || patt->filesize != fsize) {
+			const struct cli_md5m_patt *patt;
+			if(cli_md5sect(map, &exe_sections[i], md5_dig) && cli_md5m_scan(md5_dig, exe_sections[i].rsz, ctx->virname, ctx->engine->md5_mdb) == CL_VIRUS) {
+			    if(cli_md5m_scan(md5_dig, fsize, NULL, ctx->engine->md5_fp) != CL_VIRUS) {
 				free(section_hdr);
 				free(exe_sections);
 				return CL_VIRUS;
@@ -1020,7 +1024,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 	    cli_dbgmsg("Found PE values with sign bit set\n");
 	    free(section_hdr);
 	    free(exe_sections);
-	    if(DETECT_BROKEN) {
+	    if(DETECT_BROKEN_PE) {
 	        if(ctx->virname)
 		    *ctx->virname = "Heuristics.Broken.Executable";
 		return CL_VIRUS;
@@ -1029,7 +1033,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 	}
 
 	if(!i) {
-	    if (DETECT_BROKEN && exe_sections[i].urva!=hdr_size) { /* Bad first section RVA */
+	    if (DETECT_BROKEN_PE && exe_sections[i].urva!=hdr_size) { /* Bad first section RVA */
 	        cli_dbgmsg("First section is in the wrong place\n");
 	        if(ctx->virname)
 		    *ctx->virname = "Heuristics.Broken.Executable";
@@ -1040,7 +1044,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 	    min = exe_sections[i].rva;
 	    max = exe_sections[i].rva + exe_sections[i].rsz;
 	} else {
-	    if (DETECT_BROKEN && exe_sections[i].urva - exe_sections[i-1].urva != exe_sections[i-1].vsz) { /* No holes, no overlapping, no virtual disorder */
+	    if (DETECT_BROKEN_PE && exe_sections[i].urva - exe_sections[i-1].urva != exe_sections[i-1].vsz) { /* No holes, no overlapping, no virtual disorder */
 	        cli_dbgmsg("Virtually misplaced section (wrong order, overlapping, non contiguous)\n");
 	        if(ctx->virname)
 		    *ctx->virname = "Heuristics.Broken.Executable";
@@ -1063,7 +1067,7 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
     if(!(ep = cli_rawaddr(vep, exe_sections, nsections, &err, fsize, hdr_size)) && err) {
 	cli_dbgmsg("EntryPoint out of file\n");
 	free(exe_sections);
-	if(DETECT_BROKEN) {
+	if(DETECT_BROKEN_PE) {
 	    if(ctx->virname)
 		*ctx->virname = "Heuristics.Broken.Executable";
 	    return CL_VIRUS;
@@ -1073,15 +1077,6 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 
     cli_dbgmsg("EntryPoint offset: 0x%x (%d)\n", ep, ep);
 
-    if(iconset){
-	if(!dll && dirs[2].Size && cli_scanicon(iconset, EC32(dirs[2].VirtualAddress), ctx, exe_sections, nsections, hdr_size) == CL_VIRUS) {
-	    free(exe_sections);
-	    return CL_VIRUS;
-	}
-	free(exe_sections);
-	return CL_CLEAN;
-    }
-
     if(pe_plus) { /* Do not continue for PE32+ files */
 	free(exe_sections);
 	return CL_CLEAN;
@@ -1114,6 +1109,34 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 	}
     }
 
+    pedata.nsections = nsections;
+    pedata.ep = ep;
+    pedata.offset = 0;
+    memcpy(&pedata.file_hdr, &file_hdr, sizeof(file_hdr));
+    memcpy(&pedata.opt32, &pe_opt.opt32, sizeof(pe_opt.opt32));
+    memcpy(&pedata.opt64, &pe_opt.opt64, sizeof(pe_opt.opt64));
+    memcpy(&pedata.dirs, dirs, sizeof(pedata.dirs));
+    pedata.e_lfanew = e_lfanew;
+    pedata.overlays = overlays;
+    pedata.overlays_sz = fsize - overlays;
+    pedata.hdr_size = hdr_size;
+
+    /* Bytecode BC_PE_ALL hook */
+    bc_ctx = cli_bytecode_context_alloc();
+    if (!bc_ctx) {
+	cli_errmsg("cli_scanpe: can't allocate memory for bc_ctx\n");
+	return CL_EMEM;
+    }
+    cli_bytecode_context_setpe(bc_ctx, &pedata, exe_sections);
+    cli_bytecode_context_setctx(bc_ctx, ctx);
+    ret = cli_bytecode_runhook(ctx, ctx->engine, bc_ctx, BC_PE_ALL, map, ctx->virname);
+    if (ret == CL_VIRUS || ret == CL_BREAK) {
+	free(exe_sections);
+	cli_bytecode_context_destroy(bc_ctx);
+	return ret == CL_VIRUS ? CL_VIRUS : CL_CLEAN;
+    }
+    cli_bytecode_context_destroy(bc_ctx);
+
     /* Attempt to detect some popular polymorphic viruses */
 
     /* W32.Parite.B */
@@ -2248,28 +2271,19 @@ int cli_scanpe(cli_ctx *ctx, icon_groupset *iconset)
 
     /* to be continued ... */
 
-    /* Bytecode */
+    /* Bytecode BC_PE_UNPACKER hook */
     bc_ctx = cli_bytecode_context_alloc();
     if (!bc_ctx) {
 	cli_errmsg("cli_scanpe: can't allocate memory for bc_ctx\n");
 	return CL_EMEM;
     }
-    pedata.nsections = nsections;
-    pedata.ep = ep;
-    pedata.offset = 0;
-    memcpy(&pedata.file_hdr, &file_hdr, sizeof(file_hdr));
-    memcpy(&pedata.opt32, &pe_opt.opt32, sizeof(pe_opt.opt32));
-    memcpy(&pedata.opt64, &pe_opt.opt64, sizeof(pe_opt.opt64));
-    memcpy(&pedata.dirs, dirs, sizeof(pedata.dirs));
-    pedata.e_lfanew = e_lfanew;
-    pedata.overlays = overlays;
-    pedata.overlays_sz = fsize - overlays;
-    pedata.hdr_size = hdr_size;
     cli_bytecode_context_setpe(bc_ctx, &pedata, exe_sections);
     cli_bytecode_context_setctx(bc_ctx, ctx);
     ret = cli_bytecode_runhook(ctx, ctx->engine, bc_ctx, BC_PE_UNPACKER, map, ctx->virname);
     switch (ret) {
 	case CL_VIRUS:
+	    free(exe_sections);
+	    cli_bytecode_context_destroy(bc_ctx);
 	    return CL_VIRUS;
 	case CL_SUCCESS:
 	    ndesc = cli_bytecode_context_getresult_file(bc_ctx, &tempfile);
@@ -2378,7 +2392,7 @@ int cli_peheader(fmap_t *map, struct cli_exe_info *peinfo)
     valign = (pe_plus)?EC32(optional_hdr64.SectionAlignment):EC32(optional_hdr32.SectionAlignment);
     falign = (pe_plus)?EC32(optional_hdr64.FileAlignment):EC32(optional_hdr32.FileAlignment);
 
-    hdr_size = PESALIGN(hdr_size, valign);
+    peinfo->hdr_size = hdr_size = PESALIGN(hdr_size, valign);
 
     peinfo->section = (struct cli_exe_section *) cli_calloc(peinfo->nsections, sizeof(struct cli_exe_section));
 
@@ -2442,7 +2456,12 @@ int cli_peheader(fmap_t *map, struct cli_exe_info *peinfo)
 	return -1;
     }
 
-    while(dirs[2].Size && peinfo->vinfo) {
+    if(EC16(file_hdr.Characteristics) & 0x2000 || !dirs[2].Size)
+	peinfo->res_addr = 0;
+    else
+	peinfo->res_addr = EC32(dirs[2].VirtualAddress);
+
+    while(dirs[2].Size) {
 	struct vinfo_list vlist;
 	uint8_t *vptr, *baseptr;
     	uint32_t rva, res_sz;
@@ -2451,7 +2470,7 @@ int cli_peheader(fmap_t *map, struct cli_exe_info *peinfo)
 	memset(&vlist, 0, sizeof(vlist));
     	findres(0x10, 0xffffffff, EC32(dirs[2].VirtualAddress), map, peinfo->section, peinfo->nsections, hdr_size, versioninfo_cb, &vlist);
 	if(!vlist.count) break; /* No version_information */
-	if(cli_hashset_init(peinfo->vinfo, 32, 80)) {
+	if(cli_hashset_init(&peinfo->vinfo, 32, 80)) {
 	    cli_errmsg("cli_peheader: Unable to init vinfo hashset\n");
 	    free(section_hdr);
 	    free(peinfo->section);
@@ -2576,9 +2595,9 @@ int cli_peheader(fmap_t *map, struct cli_exe_info *peinfo)
 				continue;
 			    }
 
-			    if(cli_hashset_addkey(peinfo->vinfo, (uint32_t)(vptr - baseptr + 6))) {
+			    if(cli_hashset_addkey(&peinfo->vinfo, (uint32_t)(vptr - baseptr + 6))) {
 				cli_errmsg("cli_peheader: Unable to add rva to vinfo hashset\n");
-				cli_hashset_destroy(peinfo->vinfo);
+				cli_hashset_destroy(&peinfo->vinfo);
 				free(section_hdr);
 				free(peinfo->section);
 				peinfo->section = NULL;
