@@ -30,6 +30,7 @@
 
 
 #define EC32(x) le32_to_host(x)
+#define EC16(x) le16_to_host(x)
 #define USE_FLOATS
 #ifdef USE_FLOATS
 #define LABDIFF(x) labdiff(x)
@@ -46,7 +47,7 @@ struct GICONS {
 static int groupicon_cb(void *ptr, uint32_t type, uint32_t name, uint32_t lang, uint32_t rva) {
     struct GICONS *gicons = ptr;
     type = type; lang = lang;
-    cli_dbgmsg("groupicon_cb: got group %u\n", name);
+    cli_dbgmsg("groupicon_cb: got group %x\n", name);
     if(!gicons->cnt || gicons->lastg == name) {
 	gicons->rvas[gicons->cnt] = rva;
 	gicons->cnt++;
@@ -65,7 +66,7 @@ struct ICONS {
 static int icon_cb(void *ptr, uint32_t type, uint32_t name, uint32_t lang, uint32_t rva) {
     struct ICONS *icons = ptr;
     type = type; lang = lang;
-    cli_dbgmsg("icon_cb: got icon %u\n", name);
+    cli_dbgmsg("icon_cb: got icon %x\n", name);
     if(icons->cnt > 100) 
 	return 1;
     icons->rvas[icons->cnt] = rva;
@@ -102,7 +103,7 @@ int cli_scanicon(icon_groupset *set, uint32_t resdir_rva, cli_ctx *ctx, struct c
 		    uint32_t sz;
 		    uint16_t id;
 		} *dir;
-		
+
 		grp = fmap_need_off_once(map, cli_rawaddr(cli_readint32(grp), exe_sections, nsections, &err, map->len, hdr_size), gsz);
 		if(grp && !err) {
 		    icnt = cli_readint32(grp+2) >> 16;
@@ -111,8 +112,8 @@ int cli_scanicon(icon_groupset *set, uint32_t resdir_rva, cli_ctx *ctx, struct c
 
 		    while(icnt && gsz >= 14) {
 			dir = (struct icondir *)grp;
-			cli_dbgmsg("cli_scanicon: Icongrp @%x - %ux%ux%u - (id=%x, rsvd=%u, planes=%u, palcnt=%u, sz=%x)\n", gicons.rvas[curicon], dir->w, dir->h, dir->depth, dir->id, dir->planes, dir->palcnt, dir->rsvd, dir->sz);
-			findres(3, dir->id, resdir_rva, map, exe_sections, nsections, hdr_size, icon_cb, &icons);
+			cli_dbgmsg("cli_scanicon: Icongrp @%x - %ux%ux%u - (id=%x, rsvd=%u, planes=%u, palcnt=%u, sz=%x)\n", gicons.rvas[curicon], dir->w, dir->h, cli_readint16(&dir->depth), cli_readint16(&dir->id), cli_readint16(&dir->planes), dir->palcnt, dir->rsvd, cli_readint32(&dir->sz));
+			findres(3, cli_readint16(&dir->id), resdir_rva, map, exe_sections, nsections, hdr_size, icon_cb, &icons);
 			grp += 14;
 			gsz -= 14;
 		    }
@@ -1200,6 +1201,7 @@ static int parseicon(icon_groupset *set, uint32_t rva, cli_ctx *ctx, struct cli_
     fmap_t *map;
     uint32_t icoff;
     struct icon_matcher *matcher;
+    unsigned int special_32_is_32 = 0;
 
     if(!ctx || !ctx->engine || !(matcher=ctx->engine->iconcheck))
 	return CL_SUCCESS;
@@ -1227,10 +1229,10 @@ static int parseicon(icon_groupset *set, uint32_t rva, cli_ctx *ctx, struct cli_
 
     /* seek to the end of v4/v5 header */
     icoff += EC32(bmphdr.sz);
-	    
+
     width = EC32(bmphdr.w);
     height = EC32(bmphdr.h) / 2;
-    depth = EC32(bmphdr.depth);
+    depth = EC16(bmphdr.depth);
     if(width > 256 || height > 256 || width < 16 || height < 16) {
 	cli_dbgmsg("parseicon: Image too small or too big (%ux%u)\n", width, height);
 	return CL_SUCCESS;
@@ -1306,7 +1308,7 @@ static int parseicon(icon_groupset *set, uint32_t rva, cli_ctx *ctx, struct cli_
 		    have = 8;
 		}
 		have -= depth;
-		imagedata[(height - 1 - y) * width + x] = palette[(c >> have) & ((1<<depth)-1)];
+		imagedata[(height - 1 - y) * width + x] = EC32(palette[(c >> have) & ((1<<depth)-1)]);
 	    }
 	    break;
 	}
@@ -1332,7 +1334,9 @@ static int parseicon(icon_groupset *set, uint32_t rva, cli_ctx *ctx, struct cli_
 	    break;
 	case 32:
 	    for(x=0; x<width; x++) {
-		imagedata[(height - 1 - y) * width + x] = rawimage[x_off] | (rawimage[x_off + 1] << 8) | (rawimage[x_off + 2] << 16) | (rawimage[x_off + 3] << 24);
+		unsigned int a = rawimage[x_off + 3] << 24;
+		imagedata[(height - 1 - y) * width + x] = rawimage[x_off] | (rawimage[x_off + 1] << 8) | (rawimage[x_off + 2] << 16) | a;
+		special_32_is_32 |= a;
 		x_off+=4;
 	    }
 	    break;
@@ -1342,10 +1346,24 @@ static int parseicon(icon_groupset *set, uint32_t rva, cli_ctx *ctx, struct cli_
     if(palette) fmap_unneed_ptr(map, palette, (1<<depth) * sizeof(int));
     makebmp("0-noalpha", tempd, width, height, imagedata);
 
+    if(depth == 32 && !special_32_is_32) { /* Sometimes it really is 24. Exploited live - see sample 0013839101 */
+	andlinesz = 4*(width / 32) + 4*(width % 32 != 0);
+	if(!(rawimage = fmap_need_off_once(map, icoff + height * scanlinesz, height * andlinesz))) {
+	    /* Likely a broken sample - 32bit icon with 24bit data and a broken mask:
+	       i could really break out here but i've got the full image, so i'm just forcing full alpha
+	       Found in samples: 0008777448, 0009116157, 0009116157 */
+	    for(y=0; y<height; y++)
+		for(x=0; x<width; x++)
+		    imagedata[y * width + x] |= 0xff000000;
+	    special_32_is_32 = 1;
+	    cli_dbgmsg("parseicon: found a broken and stupid icon\n");
+	} else cli_dbgmsg("parseicon: found a stupid icon\n");
+    } else rawimage += height * scanlinesz;
+
     /* Set alpha on or off based on the mask */
-    if(depth & 0x1f) {
+    if((depth & 0x1f) || !special_32_is_32) {
 	for(y=0; y<height; y++) {
-	    unsigned int x_off = height * scanlinesz + y * andlinesz;
+	    unsigned int x_off = y * andlinesz;
 	    unsigned int have = 0;
 	    unsigned char c;
 	    for(x=0; x<width; x++) {
@@ -1503,13 +1521,6 @@ static int parseicon(icon_groupset *set, uint32_t rva, cli_ctx *ctx, struct cli_
     return CL_SUCCESS;
 }
 
-
-int cli_match_icon(icon_groupset *set, cli_ctx *ctx) {
-    if(!ctx || !ctx->engine || !ctx->engine->iconcheck || !ctx->engine->iconcheck->group_counts[0] || !ctx->engine->iconcheck->group_counts[1])
-	return CL_CLEAN;
-    return cli_scanpe(ctx, set);
-}
-
 void cli_icongroupset_add(const char *groupname, icon_groupset *set, unsigned int type, cli_ctx *ctx) {
     struct icon_matcher *matcher;
     unsigned int i, j;
