@@ -46,6 +46,7 @@
 #endif
 #include "matcher-ac.h"
 #include "matcher-bm.h"
+#include "matcher-md5.h"
 #include "matcher.h"
 #include "others.h"
 #include "str.h"
@@ -549,6 +550,11 @@ static int cli_loaddb(FILE *fs, struct cl_engine *engine, unsigned int *signo, u
 	if(engine->ignored && cli_chkign(engine->ignored, start, buffer_cpy))
 	    continue;
 
+	if(engine->cb_sigload && engine->cb_sigload("db", start, engine->cb_sigload_ctx)) {
+	    cli_dbgmsg("cli_loaddb: skipping %s due to callback\n", start);
+	    continue;
+	}
+
 	if(*pt == '=') continue;
 
 	if((ret = cli_parse_add(root, start, pt, 0, 0, "*", 0, NULL, options))) {
@@ -593,8 +599,10 @@ static int cli_loadidb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	return CL_EMEM;
     
     if(engine->ignored)
-	if(!(buffer_cpy = cli_malloc(FILEBUFF)))
+	if(!(buffer_cpy = cli_malloc(FILEBUFF))) {
+	    mpool_free(engine->mempool, matcher);
 	    return CL_EMEM;
+	}
 
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
@@ -621,6 +629,11 @@ static int cli_loadidb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	if(engine->ignored && cli_chkign(engine->ignored, tokens[0], buffer_cpy))
 	    continue;
 
+	if(engine->cb_sigload && engine->cb_sigload("idb", tokens[0], engine->cb_sigload_ctx)) {
+	    cli_dbgmsg("cli_loadidb: skipping %s due to callback\n", tokens[0]);
+	    continue;
+	}
+
 	hash = (uint8_t *)tokens[3];
 	if(cli_hexnibbles((char *)hash, 124)) {
 	    cli_errmsg("cli_loadidb: Malformed hash at line %u (bad chars)\n", line);
@@ -817,7 +830,7 @@ static int cli_loadwdb(FILE *fs, struct cl_engine *engine, unsigned int options,
 	}
     }
 
-    if((ret = load_regex_matcher(engine->whitelist_matcher, fs, NULL, options, 1, dbio, engine->dconf->other&OTHER_CONF_PREFILTERING))) {
+    if((ret = load_regex_matcher(engine, engine->whitelist_matcher, fs, NULL, options, 1, dbio, engine->dconf->other&OTHER_CONF_PREFILTERING))) {
 	return ret;
     }
 
@@ -838,7 +851,7 @@ static int cli_loadpdb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	}
     }
 
-    if((ret = load_regex_matcher(engine->domainlist_matcher, fs, signo, options, 0, dbio, engine->dconf->other&OTHER_CONF_PREFILTERING))) {
+    if((ret = load_regex_matcher(engine, engine->domainlist_matcher, fs, signo, options, 0, dbio, engine->dconf->other&OTHER_CONF_PREFILTERING))) {
 	return ret;
     }
 
@@ -890,6 +903,11 @@ static int cli_loadndb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	if(engine->ignored && cli_chkign(engine->ignored, virname, buffer_cpy))
 	    continue;
 
+	if(!sdb && engine->cb_sigload && engine->cb_sigload("ndb", virname, engine->cb_sigload_ctx)) {
+	    cli_dbgmsg("cli_loadndb: skipping %s due to callback\n", virname);
+	    continue;
+	}
+
 	if(tokens_count > 4) { /* min version */
 	    pt = tokens[4];
 
@@ -972,7 +990,7 @@ struct lsig_attrib {
 static int lsigattribs(char *attribs, struct cli_lsig_tdb *tdb)
 {
 	struct lsig_attrib attrtab[] = {
-#define ATTRIB_TOKENS	8
+#define ATTRIB_TOKENS	9
 	    { "Target",		    CLI_TDB_UINT,	(void **) &tdb->target	    },
 	    { "Engine",		    CLI_TDB_RANGE,	(void **) &tdb->engine	    },
 
@@ -984,6 +1002,7 @@ static int lsigattribs(char *attribs, struct cli_lsig_tdb *tdb)
 	    { "IconGroup2",	    CLI_TDB_STR,	(void **) &tdb->icongrp2    },
 
 	    { "Container",	    CLI_TDB_FTYPE,	(void **) &tdb->container   },
+	    { "HandlerType",	    CLI_TDB_FTYPE,	(void **) &tdb->handlertype },
 /*
 	    { "SectOff",    CLI_TDB_RANGE2,	(void **) &tdb->sectoff	    },
 	    { "SectRVA",    CLI_TDB_RANGE2,	(void **) &tdb->sectrva	    },
@@ -1193,6 +1212,12 @@ static int load_oneldb(char *buffer, int chkpua, int chkign, struct cl_engine *e
     if (chkign && cli_chkign(engine->ignored, virname, buffer_cpy))
 	return CL_SUCCESS;
 
+    if(engine->cb_sigload && engine->cb_sigload("ldb", virname, engine->cb_sigload_ctx)) {
+	cli_dbgmsg("cli_loadldb: skipping %s due to callback\n", virname);
+	(*sigs)--;
+	return CL_SUCCESS;
+    }
+
     subsigs = cli_ac_chklsig(logic, logic + strlen(logic), NULL, NULL, NULL, 1);
     if(subsigs == -1) {
 	return CL_EMALFDB;
@@ -1384,6 +1409,7 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
     struct cli_bc *bc;
     unsigned sigs = 0;
     unsigned security_trust = 0;
+    unsigned i;
 
 
     /* TODO: virusname have a common prefix, and whitelist by that */
@@ -1393,9 +1419,16 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
     if(!(engine->dconf->bytecode & BYTECODE_ENGINE_MASK)) {
 	return CL_SUCCESS;
     }
+
+    if(engine->cb_sigload && engine->cb_sigload("cbc", dbname, engine->cb_sigload_ctx)) {
+	cli_dbgmsg("cli_loadcbc: skipping %s due to callback\n", dbname);
+	return CL_SUCCESS;
+    }
+
 #ifndef CL_BCUNSIGNED
     if (!(options & CL_DB_SIGNED)) {
 	cli_warnmsg("Only loading signed bytecode, skipping load of unsigned bytecode!\n");
+	cli_warnmsg("Build with ./configure --enable-unsigned-bytecode to enable loading of unsigned bytecode\n");
 	return CL_SUCCESS;
     }
 #endif
@@ -1425,6 +1458,7 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
     while (cli_dbgets(buf, sizeof(buf), fs, dbio)) {}
 
     if (rc != CL_SUCCESS) {
+	cli_bytecode_destroy(bc);
 	cli_errmsg("Unable to load %s bytecode: %s\n", dbname, cl_strerror(rc));
 	return rc;
     }
@@ -1473,6 +1507,20 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	    }
 	    engine->hooks[hook][cnt-1] = bcs->count-1;
 	} else switch (bc->kind) {
+	    case BC_STARTUP:
+		for (i=0;i<bcs->count-1;i++)
+		    if (bcs->all_bcs[i].kind == BC_STARTUP) {
+			struct cli_bc *bc0 = &bcs->all_bcs[i];
+			cli_errmsg("Can only load 1 BC_STARTUP bytecode, attempted to load 2nd!\n");
+			cli_warnmsg("Previous BC_STARTUP: %d %d by %s\n",
+				    bc0->id, (uint32_t)bc0->metadata.timestamp,
+				    bc0->metadata.sigmaker ? bc0->metadata.sigmaker : "N/A");
+			cli_warnmsg("Conflicting BC_STARTUP: %d %d by %s\n",
+				    bc->id, (uint32_t)bc->metadata.timestamp,
+				    bc->metadata.sigmaker ? bc->metadata.sigmaker : "N/A");
+			return CL_EMALFDB;
+		    }
+		break;
 	    default:
 		cli_errmsg("Bytecode: unhandled bytecode kind %u\n", bc->kind);
 		return CL_EMALFDB;
@@ -1829,8 +1877,8 @@ static int cli_md5db_init(struct cl_engine *engine, unsigned int mode)
 #ifdef USE_MPOOL
     bm->mempool = engine->mempool;
 #endif
-    if((ret = cli_bm_init(bm))) {
-	cli_errmsg("cli_md5db_init: Failed to initialize B-M\n");
+    if((ret = cli_md5m_init(bm))) {
+	cli_errmsg("cli_md5db_init: Failed to initialize MD5 matcher\n");
 	return ret;
     }
 
@@ -1851,9 +1899,10 @@ static int cli_loadmd5(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	const char *tokens[MD5_TOKENS + 1];
 	char buffer[FILEBUFF], *buffer_cpy = NULL;
 	const char *pt;
+	unsigned char *md5;
 	int ret = CL_SUCCESS;
 	unsigned int size_field = 1, md5_field = 0, line = 0, sigs = 0, tokens_count;
-	struct cli_bm_patt *new;
+	struct cli_md5m_patt *new;
 	struct cli_matcher *db = NULL;
 
 
@@ -1891,26 +1940,38 @@ static int cli_loadmd5(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	if(engine->ignored && cli_chkign(engine->ignored, pt, buffer_cpy))
 	    continue;
 
-	new = (struct cli_bm_patt *) mpool_calloc(engine->mempool, 1, sizeof(struct cli_bm_patt));
+	if(engine->cb_sigload) {
+	    const char *dot = strchr(dbname, '.');
+	    if(!dot)
+		dot = dbname;
+	    else
+		dot++;
+	    if(engine->cb_sigload(dot, pt, engine->cb_sigload_ctx)) {
+		cli_dbgmsg("cli_loadmd5: skipping %s due to callback\n", pt);
+	        continue;
+	    }
+	}
+
+	new = (struct cli_md5m_patt *) mpool_calloc(engine->mempool, 1, sizeof(struct cli_md5m_patt));
 	if(!new) {
 	    ret = CL_EMEM;
 	    break;
 	}
 
 	pt = tokens[md5_field]; /* md5 */
-	if(strlen(pt) != 32 || !(new->pattern = (unsigned char *) cli_mpool_hex2str(engine->mempool, pt))) {
+	if(strlen(pt) != 32 || !(md5 = (unsigned char *) cli_mpool_hex2str(engine->mempool, pt))) {
 	    cli_errmsg("cli_loadmd5: Malformed MD5 string at line %u\n", line);
 	    mpool_free(engine->mempool, new);
 	    ret = CL_EMALFDB;
 	    break;
 	}
-	new->length = 16;
+	memcpy(new->md5, md5, 16);
+	mpool_free(engine->mempool, md5);
 
 	new->filesize = atoi(tokens[size_field]);
 
 	new->virname = cli_mpool_virname(engine->mempool, tokens[2], options & CL_DB_OFFICIAL);
 	if(!new->virname) {
-	    mpool_free(engine->mempool, new->pattern);
 	    mpool_free(engine->mempool, new);
 	    ret = CL_EMALFDB;
 	    break;
@@ -1918,7 +1979,6 @@ static int cli_loadmd5(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 
 	MD5_DB;
 	if(!db && (ret = cli_md5db_init(engine, mode))) {
-	    mpool_free(engine->mempool, new->pattern);
 	    mpool_free(engine->mempool, new->virname);
 	    mpool_free(engine->mempool, new);
 	    break;
@@ -1926,9 +1986,8 @@ static int cli_loadmd5(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	    MD5_DB;
 	}
 
-	if((ret = cli_bm_addpatt(db, new, "0"))) {
+	if((ret = cli_md5m_addpatt(db, new))) {
 	    cli_errmsg("cli_loadmd5: Error adding BM pattern\n");
-	    mpool_free(engine->mempool, new->pattern);
 	    mpool_free(engine->mempool, new->virname);
 	    mpool_free(engine->mempool, new);
 	    break;
@@ -2042,6 +2101,13 @@ static int cli_loadmd(FILE *fs, struct cl_engine *engine, unsigned int *signo, i
 	    continue;
 	}
 
+	if(engine->cb_sigload && engine->cb_sigload("md", new->virname, engine->cb_sigload_ctx)) {
+	    cli_dbgmsg("cli_loadmd: skipping %s due to callback\n", new->virname);
+	    mpool_free(engine->mempool, new->virname);
+	    mpool_free(engine->mempool, new);
+	    continue;
+	}
+
 	new->encrypted = strcmp(tokens[1], "*") ? atoi(tokens[1]) : 2;
 
 	if(strcmp(tokens[2], "*") && cli_regcomp(&new->name, tokens[2], REG_EXTENDED | REG_NOSUB)) {
@@ -2175,6 +2241,13 @@ static int cli_loadcdb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	    continue;
 	}
 
+	if(engine->cb_sigload && engine->cb_sigload("cdb", new->virname, engine->cb_sigload_ctx)) {
+	    cli_dbgmsg("cli_loadcdb: skipping %s due to callback\n", new->virname);
+	    mpool_free(engine->mempool, new->virname);
+	    mpool_free(engine->mempool, new);
+	    continue;
+	}
+
 	if(!strcmp(tokens[1], "*")) {
 	    new->ctype = CL_TYPE_ANY;
 	} else if((new->ctype = cli_ftcode(tokens[1])) == CL_TYPE_ERROR) {
@@ -2594,8 +2667,8 @@ int cl_load(const char *path, struct cl_engine *engine, unsigned int *signo, uns
 	if((ret = phishing_init(engine)))
 	    return ret;
 
-    if((dboptions & CL_DB_BYTECODE) && !engine->bcs.engine && (engine->dconf->bytecode & BYTECODE_ENGINE_MASK)) {
-	if((ret = cli_bytecode_init(&engine->bcs, engine->dconf->bytecode)))
+    if((dboptions & CL_DB_BYTECODE) && !engine->bcs.inited) {
+	if((ret = cli_bytecode_init(&engine->bcs)))
 	    return ret;
     } else {
 	cli_dbgmsg("Bytecode engine disabled\n");
@@ -2871,12 +2944,12 @@ int cl_engine_free(struct cl_engine *engine)
     }
 
     if((root = engine->md5_hdb)) {
-	cli_bm_free(root);
+	cli_md5m_free(root);
 	mpool_free(engine->mempool, root);
     }
 
     if((root = engine->md5_mdb)) {
-	cli_bm_free(root);
+	cli_md5m_free(root);
 	mpool_free(engine->mempool, root->soff);
 	if(root->md5_sizes_hs.capacity) {
 		cli_hashset_destroy(&root->md5_sizes_hs);
@@ -2885,7 +2958,7 @@ int cl_engine_free(struct cl_engine *engine)
     }
 
     if((root = engine->md5_fp)) {
-	cli_bm_free(root);
+	cli_md5m_free(root);
 	mpool_free(engine->mempool, root);
     }
 
@@ -2931,7 +3004,10 @@ int cl_engine_free(struct cl_engine *engine)
 	struct icon_matcher *iconcheck = engine->iconcheck;
 	for(i=0; i<3; i++) {
 	    if(iconcheck->icons[i]) {
-		mpool_free(engine->mempool, iconcheck->icons[i]->name);
+		for (j=0;j<iconcheck->icon_counts[i];j++) {
+		    struct icomtr* metric = iconcheck->icons[i];
+		    mpool_free(engine->mempool, metric[j].name);
+		}
 		mpool_free(engine->mempool, iconcheck->icons[i]);
 	    }
 	}
@@ -3008,9 +3084,14 @@ int cl_engine_compile(struct cl_engine *engine)
 	if((root = engine->root[i])) {
 	    if((ret = cli_ac_buildtrie(root)))
 		return ret;
-	    cli_dbgmsg("matcher[%u]: %s: AC sigs: %u (reloff: %u, absoff: %u) BM sigs: %u (reloff: %u, absoff: %u) maxpatlen %u %s\n", i, cli_mtargets[i].name, root->ac_patterns, root->ac_reloff_num, root->ac_absoff_num, root->bm_patterns, root->bm_reloff_num, root->bm_absoff_num, root->maxpatlen, root->ac_only ? "(ac_only mode)" : "");
+	    cli_dbgmsg("Matcher[%u]: %s: AC sigs: %u (reloff: %u, absoff: %u) BM sigs: %u (reloff: %u, absoff: %u) maxpatlen %u %s\n", i, cli_mtargets[i].name, root->ac_patterns, root->ac_reloff_num, root->ac_absoff_num, root->bm_patterns, root->bm_reloff_num, root->bm_absoff_num, root->maxpatlen, root->ac_only ? "(ac_only mode)" : "");
 	}
     }
+    if(engine->md5_hdb)
+	cli_dbgmsg("MD5 sigs (files): %u\n", engine->md5_hdb->md5_patterns);
+
+    if(engine->md5_mdb)
+	cli_dbgmsg("MD5 sigs (PE sections): %u\n", engine->md5_mdb->md5_patterns);
 
     if((ret = cli_build_regex_list(engine->whitelist_matcher))) {
 	    return ret;
@@ -3028,7 +3109,7 @@ int cl_engine_compile(struct cl_engine *engine)
     mpool_flush(engine->mempool);
 
     /* Compile bytecode */
-    if((ret = cli_bytecode_prepare(&engine->bcs, engine->dconf->bytecode))) {
+    if((ret = cli_bytecode_prepare2(engine, &engine->bcs, engine->dconf->bytecode))) {
 	cli_errmsg("Unable to compile/load bytecode: %s\n", cl_strerror(ret));
 	return ret;
     }
