@@ -185,7 +185,7 @@ static int cli_unrar_scanmetadata(int desc, unrar_metadata_t *metadata, cli_ctx
     if(DETECT_ENCRYPTED && metadata->encrypted) {
 	cli_dbgmsg("RAR: Encrypted files found in archive.\n");
 	lseek(desc, 0, SEEK_SET);
-	ret = cli_scandesc(desc, ctx, 0, 0, NULL, AC_SCAN_VIR);
+	ret = cli_scandesc(desc, ctx, 0, 0, NULL, AC_SCAN_VIR, NULL);
 	if(ret != CL_VIRUS) {
 	    *ctx->virname = "Heuristics.Encrypted.RAR";
 	    return CL_VIRUS;
@@ -227,7 +227,7 @@ static int cli_scanrar(int desc, cli_ctx *ctx, off_t sfx_offset, uint32_t *sfx_c
 	    cli_dbgmsg("RAR: Encrypted main header\n");
 	    if(DETECT_ENCRYPTED) {
 		lseek(desc, 0, SEEK_SET);
-		ret = cli_scandesc(desc, ctx, 0, 0, NULL, AC_SCAN_VIR);
+		ret = cli_scandesc(desc, ctx, 0, 0, NULL, AC_SCAN_VIR, NULL);
 		if(ret != CL_VIRUS)
 		    *ctx->virname = "Heuristics.Encrypted.RAR";
 		return CL_VIRUS;
@@ -705,6 +705,7 @@ static int cli_scanmscab(int desc, cli_ctx *ctx, off_t sfx_offset)
 	unsigned int files = 0;
 	struct cab_archive cab;
 	struct cab_file *file;
+	unsigned int corrupted_input;
 
 
     cli_dbgmsg("in cli_scanmscab()\n");
@@ -740,13 +741,16 @@ static int cli_scanmscab(int desc, cli_ctx *ctx, off_t sfx_offset)
 	if((ret = cab_extract(file, tempname))) {
 	    cli_dbgmsg("CAB: Failed to extract file: %s\n", cl_strerror(ret));
 	} else {
-	    if(file->length != file->written_size)
+	    corrupted_input = ctx->corrupted_input;
+	    if(file->length != file->written_size) {
 		cli_dbgmsg("CAB: Length from header %u but wrote %u bytes\n", (unsigned int) file->length, (unsigned int) file->written_size);
-
+		ctx->corrupted_input = 1;
+	    }
 	    ret = cli_scanfile(tempname, ctx);
+	    ctx->corrupted_input = corrupted_input;
 	}
 	if(!ctx->engine->keeptmp) {
-	    if (cli_unlink(tempname)) {
+	    if (!access(tempname, R_OK) && cli_unlink(tempname)) {
 	    	free(tempname);
 		ret = CL_EUNLINK;
 		break;
@@ -976,7 +980,7 @@ static int cli_scanhtml(cli_ctx *ctx)
     snprintf(fullname, 1024, "%s"PATHSEP"nocomment.html", tempname);
     fd = open(fullname, O_RDONLY|O_BINARY);
     if (fd >= 0) {
-	    ret = cli_scandesc(fd, ctx, CL_TYPE_HTML, 0, NULL, AC_SCAN_VIR);
+	    ret = cli_scandesc(fd, ctx, CL_TYPE_HTML, 0, NULL, AC_SCAN_VIR, NULL);
 	    close(fd);
     }
 
@@ -986,7 +990,7 @@ static int cli_scanhtml(cli_ctx *ctx)
 	    snprintf(fullname, 1024, "%s"PATHSEP"notags.html", tempname);
 	    fd = open(fullname, O_RDONLY|O_BINARY);
 	    if(fd >= 0) {
-		    ret = cli_scandesc(fd, ctx, CL_TYPE_HTML, 0, NULL, AC_SCAN_VIR);
+		    ret = cli_scandesc(fd, ctx, CL_TYPE_HTML, 0, NULL, AC_SCAN_VIR, NULL);
 		    close(fd);
 	    }
     }
@@ -995,10 +999,10 @@ static int cli_scanhtml(cli_ctx *ctx)
 	    snprintf(fullname, 1024, "%s"PATHSEP"javascript", tempname);
 	    fd = open(fullname, O_RDONLY|O_BINARY);
 	    if(fd >= 0) {
-		    ret = cli_scandesc(fd, ctx, CL_TYPE_HTML, 0, NULL, AC_SCAN_VIR);
+		    ret = cli_scandesc(fd, ctx, CL_TYPE_HTML, 0, NULL, AC_SCAN_VIR, NULL);
 		    if (ret == CL_CLEAN) {
 			    lseek(fd, 0, SEEK_SET);
-			    ret = cli_scandesc(fd, ctx, CL_TYPE_TEXT_ASCII, 0, NULL, AC_SCAN_VIR);
+			    ret = cli_scandesc(fd, ctx, CL_TYPE_TEXT_ASCII, 0, NULL, AC_SCAN_VIR, NULL);
 		    }
 		    close(fd);
 	    }
@@ -1103,9 +1107,9 @@ static int cli_scanscript(cli_ctx *ctx)
 	}
 	free(normalized);
 	if(ret != CL_VIRUS) {
-	    ret = cli_lsig_eval(ctx, troot, &tmdata);
+	    ret = cli_lsig_eval(ctx, troot, &tmdata, NULL);
 	    if(ret != CL_VIRUS)
-		ret = cli_lsig_eval(ctx, groot, &gmdata);
+		ret = cli_lsig_eval(ctx, groot, &gmdata, NULL);
 	}
 	cli_ac_freedata(&tmdata);
 	cli_ac_freedata(&gmdata);
@@ -1625,12 +1629,13 @@ static int cli_scan_structured(int desc, cli_ctx *ctx)
     return CL_CLEAN;
 }
 
-static int cli_scanembpe(int desc, cli_ctx *ctx)
+static int cli_scanembpe(cli_ctx *ctx, off_t offset)
 {
 	int fd, bytes, ret = CL_CLEAN;
-	unsigned long int size = 0;
-	char buff[512];
+	unsigned long int size = 0, todo;
+	char *buff;
 	char *tmpname;
+	fmap_t *map = *ctx->fmap;
 
     tmpname = cli_gentemp(ctx->engine->tmpdir);
     if(!tmpname)
@@ -1642,10 +1647,27 @@ static int cli_scanembpe(int desc, cli_ctx *ctx)
 	return CL_ECREAT;
     }
 
-    while((bytes = read(desc, buff, sizeof(buff))) > 0) {
-	size += bytes;
+    todo = map->len - offset;
+    while(1) {
+	bytes = MIN(todo, map->pgsz);
+	if(!bytes)
+	    break;
 
-	if(cli_checklimits("cli_scanembpe", ctx, size + sizeof(buff), 0, 0)!=CL_CLEAN)
+	if(!(buff = fmap_need_off_once(map, offset + size, bytes))) {
+	    close(fd);
+	    if(!ctx->engine->keeptmp) {
+		if (cli_unlink(tmpname)) {
+		    free(tmpname);
+		    return CL_EUNLINK;
+		}
+	    }
+	    free(tmpname);
+	    return CL_EREAD;
+	}
+	size += bytes;
+	todo -= bytes;
+
+	if(cli_checklimits("cli_scanembpe", ctx, size, 0, 0)!=CL_CLEAN)
 	    break;
 
 	if(cli_writen(fd, buff, bytes) != bytes) {
@@ -1657,7 +1679,7 @@ static int cli_scanembpe(int desc, cli_ctx *ctx)
 		    return CL_EUNLINK;
 		}
 	    }
-	    free(tmpname);	
+	    free(tmpname);
 	    return CL_EWRITE;
 	}
     }
@@ -1709,21 +1731,10 @@ static int cli_scanraw(cli_ctx *ctx, cli_file_t type, uint8_t typercg, cli_file_
     if(typercg)
 	acmode |= AC_SCAN_FT;
 
-    ret = cli_fmap_scandesc(ctx, type == CL_TYPE_TEXT_ASCII ? 0 : type, 0, &ftoffset, acmode, refhash);
+    ret = cli_fmap_scandesc(ctx, type == CL_TYPE_TEXT_ASCII ? 0 : type, 0, &ftoffset, acmode, NULL, refhash);
 
     if(ret >= CL_TYPENO) {
 	ctx->recursion++;
-
-/*
-	if(type == CL_TYPE_TEXT_ASCII) {
-	    lseek(desc, 0, SEEK_SET);
-
-	    nret = cli_scandesc(desc, ctx, 0, ret, 1, NULL);
-	    if(nret == CL_VIRUS)
-		cli_dbgmsg("%s found in descriptor %d when scanning file type %u\n", *ctx->virname, desc, ret);
-	}
-*/
-
 	if(nret != CL_VIRUS) {
 	    lastzip = lastrar = 0xdeadbeef;
 	    fpt = ftoffset;
@@ -1778,7 +1789,7 @@ static int cli_scanraw(cli_ctx *ctx, cli_file_t type, uint8_t typercg, cli_file_
 			    ctx->container_type = CL_TYPE_AUTOIT;
 			    ctx->container_size = map->len - fpt->offset; /* not precise */
 			    cli_dbgmsg("AUTOIT signature found at %u\n", (unsigned int) fpt->offset);
-			    nret = cli_scanautoit(map->fd, ctx, fpt->offset + 23);
+			    nret = cli_scanautoit(ctx, fpt->offset + 23);
 			}
 			break;
 
@@ -1808,14 +1819,12 @@ static int cli_scanraw(cli_ctx *ctx, cli_file_t type, uint8_t typercg, cli_file_
 			    ctx->container_size = map->len - fpt->offset; /* not precise */
 			    memset(&peinfo, 0, sizeof(struct cli_exe_info));
 			    peinfo.offset = fpt->offset;
-			    lseek(map->fd, fpt->offset, SEEK_SET);
 			    if(cli_peheader(map, &peinfo) == 0) {
 				cli_dbgmsg("*** Detected embedded PE file at %u ***\n", (unsigned int) fpt->offset);
 				if(peinfo.section)
 				    free(peinfo.section);
 
-				lseek(map->fd, fpt->offset, SEEK_SET);
-				nret = cli_scanembpe(map->fd, ctx);
+				nret = cli_scanembpe(ctx, fpt->offset);
 				break_loop = 1; /* we can stop here and other
 						 * embedded executables will
 						 * be found recursively
@@ -1890,15 +1899,31 @@ static void emax_reached(cli_ctx *ctx) {
 #define LINESTR(x) #x
 #define LINESTR2(x) LINESTR(x)
 #define __AT__  " at line "LINESTR2(__LINE__)
-#define ret_from_magicscan(retcode) do {					\
-    cli_dbgmsg("cli_magic_scandesc: returning %d %s\n", retcode, __AT__);	\
-    return retcode;							\
+#define ret_from_magicscan(retcode) do {							\
+    cli_dbgmsg("cli_magic_scandesc: returning %d %s\n", retcode, __AT__);			\
+    if(ctx->engine->cb_post_scan) {								\
+	switch(ctx->engine->cb_post_scan(desc, retcode, retcode == CL_VIRUS && ctx->virname ? *ctx->virname : NULL, ctx->cb_ctx)) {		\
+	case CL_BREAK:										\
+	    cli_dbgmsg("cli_magic_scandesc: file whitelisted by callback\n");			\
+	    return CL_CLEAN;									\
+	case CL_VIRUS:										\
+	    cli_dbgmsg("cli_magic_scandesc: file blacklisted by callback\n");			\
+	    if(ctx->virname)									\
+		*ctx->virname = "Detected.By.Callback";						\
+	    return CL_VIRUS;									\
+	case CL_CLEAN:										\
+	    break;										\
+	default:										\
+	    cli_warnmsg("cli_magic_scandesc: ignoring bad return code from callback\n");	\
+	}											\
+    }												\
+    return retcode;										\
     } while(0)
 
-int cli_magic_scandesc(int desc, cli_ctx *ctx)
+static int magic_scandesc(int desc, cli_ctx *ctx, cli_file_t type)
 {
 	int ret = CL_CLEAN;
-	cli_file_t type, dettype = 0;
+	cli_file_t dettype = 0;
 	struct stat sb;
 	uint8_t typercg = 1;
 	cli_file_t current_container_type = ctx->container_type;
@@ -1949,6 +1974,27 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
 	ret_from_magicscan(CL_EMEM);
     }
 
+    if(ctx->engine->cb_pre_scan) {
+	switch(ctx->engine->cb_pre_scan(desc, ctx->cb_ctx)) {
+	case CL_BREAK:
+	    cli_dbgmsg("cli_magic_scandesc: file whitelisted by callback\n");
+	    funmap(*ctx->fmap);
+	    ctx->fmap--;
+	    ret_from_magicscan(CL_CLEAN);
+	case CL_VIRUS:
+	    cli_dbgmsg("cli_magic_scandesc: file blacklisted by callback\n");
+	    if(ctx->virname)
+		*ctx->virname = "Detected.By.Callback";
+	    funmap(*ctx->fmap);
+	    ctx->fmap--;
+	    ret_from_magicscan(CL_VIRUS);
+	case CL_CLEAN:
+	    break;
+	default:
+	    cli_warnmsg("cli_magic_scandesc: ignoring bad return code from callback\n");
+	}
+    }
+
     if(cache_check(hash, ctx) == CL_CLEAN) {
 	funmap(*ctx->fmap);
 	ctx->fmap--;
@@ -1964,7 +2010,7 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
 	else
 	    cli_dbgmsg("Raw mode: No support for special files\n");
 
-	if((ret = cli_fmap_scandesc(ctx, 0, 0, NULL, AC_SCAN_VIR, hash)) == CL_VIRUS)
+	if((ret = cli_fmap_scandesc(ctx, 0, 0, NULL, AC_SCAN_VIR, NULL, hash)) == CL_VIRUS)
 	    cli_dbgmsg("%s found in descriptor %d\n", *ctx->virname, desc);
 	else if(ret == CL_CLEAN) {
 	    if(ctx->recursion != ctx->engine->maxreclevel)
@@ -1979,7 +2025,8 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
 	ret_from_magicscan(ret);
     }
 
-    type = cli_filetype2(*ctx->fmap, ctx->engine); /* FIXMEFMAP: port to fmap */
+    if(type == CL_TYPE_ANY)
+	type = cli_filetype2(*ctx->fmap, ctx->engine); /* FIXMEFMAP: port to fmap */
     if(type == CL_TYPE_ERROR) {
 	cli_dbgmsg("cli_magic_scandesc: cli_filetype2 returned CL_TYPE_ERROR\n");
 	funmap(*ctx->fmap);
@@ -1996,6 +2043,7 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
     ctx->hook_lsig_matches = cli_bitset_init();
     if (!ctx->hook_lsig_matches) {
 	ctx->hook_lsig_matches = old_hook_lsig_matches;
+	ctx->fmap--;
 	ret_from_magicscan(CL_EMEM);
     }
 
@@ -2058,7 +2106,7 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
 	    ctx->container_type = CL_TYPE_AUTOIT;
 	    ctx->container_size = sb.st_size;
 	    if(SCAN_ARCHIVE && (DCONF_ARCH & ARCH_CONF_AUTOIT))
-		ret = cli_scanautoit(desc, ctx, 23);
+		ret = cli_scanautoit(ctx, 23);
 	    break;
 
 	case CL_TYPE_MSSZDD:
@@ -2290,7 +2338,7 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
 	        ret = cli_scanscript(ctx);
 	    if(ret != CL_VIRUS && ctx->container_type == CL_TYPE_MAIL) {
 		lseek(desc, 0, SEEK_SET);
-		ret = cli_scandesc(desc, ctx, CL_TYPE_MAIL, 0, NULL, AC_SCAN_VIR);
+		ret = cli_scandesc(desc, ctx, CL_TYPE_MAIL, 0, NULL, AC_SCAN_VIR, NULL);
 	    }
 	    break;
 	/* Due to performance reasons all executables were first scanned
@@ -2298,7 +2346,7 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
 	 */
 	case CL_TYPE_MSEXE:
 	    if(SCAN_PE && ctx->dconf->pe)
-		ret = cli_scanpe(ctx, NULL);
+		ret = cli_scanpe(ctx);
 	    break;
 	default:
 	    break;
@@ -2326,7 +2374,74 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
     }
 }
 
+int cli_magic_scandesc(int desc, cli_ctx *ctx)
+{
+    return magic_scandesc(desc, ctx, CL_TYPE_ANY);
+}
+
+int cli_magic_scandesc_type(int desc, cli_ctx *ctx, cli_file_t type)
+{
+    return magic_scandesc(desc, ctx, type);
+}
+
+int cli_scandesc_stats(int desc, const char **virname, char *virhash, unsigned int *virsize, unsigned long int *scanned, const struct cl_engine *engine, unsigned int scanoptions)
+{
+    cli_ctx ctx;
+    int rc;
+
+    memset(&ctx, '\0', sizeof(cli_ctx));
+    ctx.engine = engine;
+    ctx.virname = virname;
+    if(virsize) {
+	*virsize = 0;
+	ctx.virsize = virsize;
+	ctx.virhash = virhash;
+    }
+    ctx.scanned = scanned;
+    ctx.options = scanoptions;
+    ctx.found_possibly_unwanted = 0;
+    ctx.container_type = CL_TYPE_ANY;
+    ctx.container_size = 0;
+    ctx.dconf = (struct cli_dconf *) engine->dconf;
+    ctx.fmap = cli_calloc(sizeof(fmap_t *), ctx.engine->maxreclevel + 2);
+    if(!ctx.fmap)
+	return CL_EMEM;
+    if (!(ctx.hook_lsig_matches = cli_bitset_init())) {
+	free(ctx.fmap);
+	return CL_EMEM;
+    }
+
+#ifdef HAVE__INTERNAL__SHA_COLLECT
+    if(scanoptions & CL_SCAN_INTERNAL_COLLECT_SHA) {
+	char link[32];
+	ssize_t linksz;
+
+	snprintf(link, sizeof(link), "/proc/self/fd/%u", desc);
+	link[sizeof(link)-1]='\0';
+	if((linksz=readlink(link, ctx.entry_filename, sizeof(ctx.entry_filename)))==-1) {
+	    cli_errmsg("failed to resolve filename for descriptor %d (%s)\n", desc, link);
+	    strcpy(ctx.entry_filename, "NO_IDEA");
+	} else
+	    ctx.entry_filename[linksz]='\0';
+    } while(0);
+#endif
+
+    rc = cli_magic_scandesc(desc, &ctx);
+
+    cli_bitset_free(ctx.hook_lsig_matches);
+    free(ctx.fmap);
+    if(rc == CL_CLEAN && ctx.found_possibly_unwanted)
+    	rc = CL_VIRUS;
+    return rc;
+}
+
 int cl_scandesc(int desc, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, unsigned int scanoptions)
+{
+    return cli_scandesc_stats(desc, virname, NULL, NULL, scanned, engine, scanoptions);
+}
+
+
+int cl_scandesc_callback(int desc, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, unsigned int scanoptions, void *context)
 {
     cli_ctx ctx;
     int rc;
@@ -2340,10 +2455,14 @@ int cl_scandesc(int desc, const char **virname, unsigned long int *scanned, cons
     ctx.container_type = CL_TYPE_ANY;
     ctx.container_size = 0;
     ctx.dconf = (struct cli_dconf *) engine->dconf;
+    ctx.cb_ctx = context;
     ctx.fmap = cli_calloc(sizeof(fmap_t *), ctx.engine->maxreclevel + 2);
     if(!ctx.fmap)
 	return CL_EMEM;
-    ctx.hook_lsig_matches = cli_bitset_init();
+    if (!(ctx.hook_lsig_matches = cli_bitset_init())) {
+	free(ctx.fmap);
+	return CL_EMEM;
+    }
 
 #ifdef HAVE__INTERNAL__SHA_COLLECT
     if(scanoptions & CL_SCAN_INTERNAL_COLLECT_SHA) {
@@ -2418,6 +2537,19 @@ int cl_scanfile(const char *filename, const char **virname, unsigned long int *s
     return ret;
 }
 
+int cli_scanfile_stats(const char *filename, const char **virname, char *virhash, unsigned int *virsize, unsigned long int *scanned, const struct cl_engine *engine, unsigned int scanoptions)
+{
+	int fd, ret;
+
+    if((fd = safe_open(filename, O_RDONLY|O_BINARY)) == -1)
+	return CL_EOPEN;
+
+    ret = cli_scandesc_stats(fd, virname, virhash, virsize, scanned, engine, scanoptions);
+    close(fd);
+
+    return ret;
+}
+
 /*
 Local Variables:
    c-basic-offset: 4
