@@ -71,12 +71,6 @@
 
 #define MAX_DEL_LOOKAHEAD   200
 
-/*
- * Force backward compatibility with the cdiff interpreter of clamav < 0.95
- */
-#define COMPATIBILITY_LIMIT 980
-
-
 static const struct dblist_s {
     const char *name;
     unsigned int count;
@@ -269,25 +263,6 @@ static int utf16decode(const struct optstruct *opts)
     return 0;
 }
 
-static unsigned int countlines(const char *filename)
-{
-	FILE *fh;
-	char buff[1024];
-	unsigned int lines = 0;
-
-
-    if((fh = fopen(filename, "r")) == NULL)
-	return 0;
-
-    while(fgets(buff, sizeof(buff), fh)) {
-	if(buff[0] == '#') continue;
-	lines++;
-    }
-
-    fclose(fh);
-    return lines;
-}
-
 static char *getdsig(const char *host, const char *user, const unsigned char *data, unsigned int datalen, unsigned short mode)
 {
 	char buff[512], cmd[128], pass[30], *pt;
@@ -640,7 +615,7 @@ static int build(const struct optstruct *opts)
 	struct stat foo;
 	unsigned char buffer[FILEBUFF];
 	char *tarfile, header[513], smbuff[32], builder[32], *pt, olddb[512], patch[32], broken[32];
-	const char *dbname, *newcvd;
+	const char *dbname, *newcvd, *localdbdir = NULL;
         struct cl_engine *engine;
 	FILE *cvd, *fh;
 	gzFile *tar;
@@ -664,6 +639,9 @@ static int build(const struct optstruct *opts)
 	return -1;
     }
 
+    if(optget(opts, "datadir")->active)
+	localdbdir = optget(opts, "datadir")->strarg;
+
     if(stat("COPYING", &foo) == -1) {
 	mprintf("!build: COPYING file not found in current working directory.\n");
 	return -1;
@@ -768,9 +746,9 @@ static int build(const struct optstruct *opts)
 
     } else {
 	pt = freshdbdir();
-	snprintf(olddb, sizeof(olddb), "%s"PATHSEP"%s.cvd", pt, dbname);
+	snprintf(olddb, sizeof(olddb), "%s"PATHSEP"%s.cvd", localdbdir ? localdbdir : pt, dbname);
 	if(access(olddb, R_OK))
-	    snprintf(olddb, sizeof(olddb), "%s"PATHSEP"%s.cld", pt, dbname);
+	    snprintf(olddb, sizeof(olddb), "%s"PATHSEP"%s.cld", localdbdir ? localdbdir : pt, dbname);
 	free(pt);
     }
 
@@ -1086,15 +1064,18 @@ static int build(const struct optstruct *opts)
 static int unpack(const struct optstruct *opts)
 {
 	char name[512], *dbdir;
+	const char *localdbdir = NULL;
 
+    if(optget(opts, "datadir")->active)
+	localdbdir = optget(opts, "datadir")->strarg;
 
     if(optget(opts, "unpack-current")->enabled) {
 	dbdir = freshdbdir();
-	snprintf(name, sizeof(name), "%s"PATHSEP"%s.cvd", dbdir, optget(opts, "unpack-current")->strarg);
+	snprintf(name, sizeof(name), "%s"PATHSEP"%s.cvd", localdbdir ? localdbdir : dbdir, optget(opts, "unpack-current")->strarg);
 	if(access(name, R_OK)) {
-	    snprintf(name, sizeof(name), "%s"PATHSEP"%s.cld", dbdir, optget(opts, "unpack-current")->strarg);
+	    snprintf(name, sizeof(name), "%s"PATHSEP"%s.cld", localdbdir ? localdbdir : dbdir, optget(opts, "unpack-current")->strarg);
 	    if(access(name, R_OK)) {
-		mprintf("!unpack: Couldn't find %s CLD/CVD database\n", optget(opts, "unpack-current")->strarg);
+		mprintf("!unpack: Couldn't find %s CLD/CVD database in %s\n", optget(opts, "unpack-current")->strarg, localdbdir ? localdbdir : dbdir);
 		free(dbdir);
 		return -1;
 	    }
@@ -1213,6 +1194,7 @@ static int listdb(const char *filename, const regex_t *regex)
 {
 	FILE *fh;
 	char *buffer, *pt, *start, *dir;
+	const char *dbname, *pathsep = PATHSEP;
 	unsigned int line = 0;
 
 
@@ -1272,12 +1254,21 @@ static int listdb(const char *filename, const regex_t *regex)
 	return 0;
     }
 
+    if(!(dbname = strrchr(filename, *pathsep))) {
+	mprintf("!listdb: Invalid filename %s\n", filename);
+	fclose(fh);
+	free(buffer);
+	return -1;
+    }
+    dbname++;
+
     if(cli_strbcasestr(filename, ".db")) { /* old style database */
 
 	while(fgets(buffer, FILEBUFF, fh)) {
 	    if(regex) {
+		cli_chomp(buffer);
 		if(!cli_regexec(regex, buffer, 0, NULL, 0))
-		    mprintf("%s", buffer);
+		    mprintf("[%s] %s\n", dbname, buffer);
 		continue;
 	    }
 	    line++;
@@ -1301,13 +1292,13 @@ static int listdb(const char *filename, const regex_t *regex)
     } else if(cli_strbcasestr(filename, ".hdb") || cli_strbcasestr(filename, ".hdu") || cli_strbcasestr(filename, ".mdb") || cli_strbcasestr(filename, ".mdu")) { /* hash database */
 
 	while(fgets(buffer, FILEBUFF, fh)) {
+	    cli_chomp(buffer);
 	    if(regex) {
 		if(!cli_regexec(regex, buffer, 0, NULL, 0))
-		    mprintf("%s", buffer);
+		    mprintf("[%s] %s\n", dbname, buffer);
 		continue;
 	    }
 	    line++;
-	    cli_chomp(buffer);
 	    start = cli_strtok(buffer, 2, ":");
 
 	    if(!start) {
@@ -1327,13 +1318,13 @@ static int listdb(const char *filename, const regex_t *regex)
     } else if(cli_strbcasestr(filename, ".ndb") || cli_strbcasestr(filename, ".ndu") || cli_strbcasestr(filename, ".ldb") || cli_strbcasestr(filename, ".ldu") || cli_strbcasestr(filename, ".sdb") || cli_strbcasestr(filename, ".zmd") || cli_strbcasestr(filename, ".rmd")) {
 
 	while(fgets(buffer, FILEBUFF, fh)) {
+	    cli_chomp(buffer);
 	    if(regex) {
 		if(!cli_regexec(regex, buffer, 0, NULL, 0))
-		    mprintf("%s", buffer);
+		    mprintf("[%s] %s\n", dbname, buffer);
 		continue;
 	    }
 	    line++;
-	    cli_chomp(buffer);
 
 	    if(cli_strbcasestr(filename, ".ldb") || cli_strbcasestr(filename, ".ldu"))
 		start = cli_strtok(buffer, 0, ";");
@@ -1366,10 +1357,15 @@ static int listsigs(const struct optstruct *opts, int mode)
 	char *dbdir;
 	struct stat sb;
 	regex_t reg;
+	const char *localdbdir = NULL;
 
+    if(optget(opts, "datadir")->active)
+	localdbdir = optget(opts, "datadir")->strarg;
 
     if(mode == 0) {
 	name = optget(opts, "list-sigs")->strarg;
+	if(access(name, R_OK) && localdbdir)
+	    name = localdbdir;
 	if(stat(name, &sb) == -1) {
 	    mprintf("--list-sigs: Can't get status of %s\n", name);
 	    return -1;
@@ -1379,7 +1375,7 @@ static int listsigs(const struct optstruct *opts, int mode)
 	if(S_ISDIR(sb.st_mode)) {
 	    if(!strcmp(name, DATADIR)) {
 		dbdir = freshdbdir();
-		ret = listdir(dbdir, NULL);
+		ret = listdir(localdbdir ? localdbdir : dbdir, NULL);
 		free(dbdir);
 	    } else {
 		ret = listdir(name, NULL);
@@ -1395,7 +1391,7 @@ static int listsigs(const struct optstruct *opts, int mode)
 	}
 	mprintf_stdout = 1;
 	dbdir = freshdbdir();
-	ret = listdir(dbdir, &reg);
+	ret = listdir(localdbdir ? localdbdir : dbdir, &reg);
 	free(dbdir);
 	cli_regfree(&reg);
     }
@@ -1537,14 +1533,42 @@ static int rundiff(const struct optstruct *opts)
     return ret;
 }
 
+static int maxlinelen(const char *file)
+{
+	int fd, bytes, n = 0, nmax = 0, i;
+	char buff[512];
+
+    if((fd = open(file, O_RDONLY)) == -1) {
+	mprintf("!maxlinelen: Can't open file %s\n", file);
+	return -1;
+    }
+
+    while((bytes = read(fd, buff, 512)) > 0) {
+	for(i = 0; i < bytes; i++, ++n) {
+	    if(buff[i] == '\n') {
+		if(n > nmax)
+		    nmax = n;
+		n = 0;
+	    }
+	}
+    }
+
+    if(bytes == -1) {
+	mprintf("!maxlinelen: Can't read file %s\n", file);
+	return -1;
+    }
+
+    return nmax;
+}
+
 static int compare(const char *oldpath, const char *newpath, FILE *diff)
 {
 	FILE *old, *new;
-	char obuff[CLI_DEFAULT_LSIG_BUFSIZE + 1], nbuff[CLI_DEFAULT_LSIG_BUFSIZE + 1], tbuff[CLI_DEFAULT_LSIG_BUFSIZE + 1], *pt, *omd5, *nmd5;
+	char *obuff, *nbuff, *tbuff, *pt, *omd5, *nmd5;
 	unsigned int oline = 0, tline, found, i;
+	int l1 = 0, l2;
 	long opos;
 
-
     if(!access(oldpath, R_OK) && (omd5 = cli_md5file(oldpath))) {
 	if(!(nmd5 = cli_md5file(newpath))) {
 	    mprintf("!compare: Can't get MD5 checksum of %s\n", newpath);
@@ -1558,30 +1582,64 @@ static int compare(const char *oldpath, const char *newpath, FILE *diff)
 	}
 	free(omd5);
 	free(nmd5);
+	l1 = maxlinelen(oldpath);
     }
 
+    l2 = maxlinelen(newpath);
+    if(l1 == -1 || l2 == -1)
+	return -1;
+    l1 = MAX(l1, l2) + 1;
+
+    obuff = malloc(l1);
+    if(!obuff) {
+	mprintf("!compare: Can't allocate memory for 'obuff'\n");
+	return -1;
+    }
+    nbuff = malloc(l1);
+    if(!nbuff) {
+	mprintf("!compare: Can't allocate memory for 'nbuff'\n");
+	free(obuff);
+	return -1;
+    }
+    tbuff = malloc(l1);
+    if(!tbuff) {
+	mprintf("!compare: Can't allocate memory for 'tbuff'\n");
+	free(obuff);
+	free(nbuff);
+	return -1;
+    }
+
+    if(l1 > CLI_DEFAULT_LSIG_BUFSIZE)
+	fprintf(diff, "#LSIZE %u\n", l1 + 32);
+
     fprintf(diff, "OPEN %s\n", newpath);
 
     if(!(new = fopen(newpath, "r"))) {
 	mprintf("!compare: Can't open file %s for reading\n", newpath);
+	free(obuff);
+	free(nbuff);
+	free(tbuff);
 	return -1;
     }
     old = fopen(oldpath, "r");
 
-    while(fgets(nbuff, sizeof(nbuff), new)) {
+    while(fgets(nbuff, l1, new)) {
 	i = strlen(nbuff);
 	if(i >= 2 && (nbuff[i - 1] == '\r' || (nbuff[i - 1] == '\n' && nbuff[i - 2] == '\r'))) {
 	    mprintf("!compare: New %s file contains lines terminated with CRLF or CR\n", newpath);
 	    if(old)
 		fclose(old);
 	    fclose(new);
+	    free(obuff);
+	    free(nbuff);
+	    free(tbuff);
 	    return -1;
 	}
 	cli_chomp(nbuff);
 	if(!old) {
 	    fprintf(diff, "ADD %s\n", nbuff);
 	} else {
-	    if(fgets(obuff, sizeof(obuff), old)) {
+	    if(fgets(obuff, l1, old)) {
 		oline++;
 		cli_chomp(obuff);
 		if(!strcmp(nbuff, obuff)) {
@@ -1590,7 +1648,7 @@ static int compare(const char *oldpath, const char *newpath, FILE *diff)
 		    tline = 0;
 		    found = 0;
 		    opos = ftell(old);
-		    while(fgets(tbuff, sizeof(tbuff), old)) {
+		    while(fgets(tbuff, l1, old)) {
 			tline++;
 			cli_chomp(tbuff);
 
@@ -1612,7 +1670,7 @@ static int compare(const char *oldpath, const char *newpath, FILE *diff)
 			    if((pt = strchr(tbuff, ' ')))
 				*pt = 0;
 			    fprintf(diff, "DEL %u %s\n", oline + i, tbuff);
-			    if(!fgets(tbuff, sizeof(tbuff), old))
+			    if(!fgets(tbuff, l1, old))
 				break;
 			}
 			oline += tline;
@@ -1630,20 +1688,11 @@ static int compare(const char *oldpath, const char *newpath, FILE *diff)
 		fprintf(diff, "ADD %s\n", nbuff);
 	    }
 	}
-#ifdef COMPATIBILITY_LIMIT
-       if(!cli_strbcasestr(newpath, ".cbc") && strlen(nbuff) > COMPATIBILITY_LIMIT) {
-	   mprintf("!compare: COMPATIBILITY_LIMIT: Found too long line in new %s\n", newpath);
-           if(old)
-               fclose(old);
-	   fclose(new);
-           return -1;
-       }
-#endif
     }
     fclose(new);
 
     if(old) {
-	while(fgets(obuff, sizeof(obuff), old)) {
+	while(fgets(obuff, l1, old)) {
 	    oline++;
 	    obuff[16] = 0;
 	    if((pt = strchr(obuff, ' ')))
@@ -1654,6 +1703,9 @@ static int compare(const char *oldpath, const char *newpath, FILE *diff)
     }
 
     fprintf(diff, "CLOSE\n");
+    free(obuff);
+    free(nbuff);
+    free(tbuff);
     return 0;
 }
 
@@ -1793,39 +1845,79 @@ static int verifydiff(const char *diff, const char *cvd, const char *incdir)
     return ret;
 }
 
-static int matchsig(const char *sig, int fd)
+static void matchsig(const char *sig, int fd)
 {
 	struct cl_engine *engine;
+        struct cli_ac_result *acres = NULL, *res;
+	struct stat sb;
+	unsigned int matches = 0;
+	cli_ctx ctx;
 	int ret;
 
     if(!(engine = cl_engine_new())) {
 	mprintf("!matchsig: Can't create new engine\n");
-	return 0;
+	return;
     }
+    cl_engine_set_num(engine, CL_ENGINE_AC_ONLY, 1);
 
     if(cli_initroots(engine, 0) != CL_SUCCESS) {
 	mprintf("!matchsig: cli_initroots() failed\n");
 	cl_engine_free(engine);
-	return 0;
+	return;
     }
 
     if(cli_parse_add(engine->root[0], "test", sig, 0, 0, "*", 0, NULL, 0) != CL_SUCCESS) {
 	mprintf("!matchsig: Can't parse signature\n");
 	cl_engine_free(engine);
-	return 0;
+	return;
     }
 
     if(cl_engine_compile(engine) != CL_SUCCESS) {
 	mprintf("!matchsig: Can't compile engine\n");
 	cl_engine_free(engine);
-	return 0;
+	return;
+    }
+    memset(&ctx, '\0', sizeof(cli_ctx));
+    ctx.engine = engine;
+    ctx.options = CL_SCAN_STDOPT;
+    ctx.container_type = CL_TYPE_ANY;
+    ctx.dconf = (struct cli_dconf *) engine->dconf;
+    ctx.fmap = calloc(sizeof(fmap_t *), 1);
+    if(!ctx.fmap) {
+	cl_engine_free(engine);
+	return;
     }
-
     lseek(fd, 0, SEEK_SET);
-    ret = cl_scandesc(fd, NULL, NULL, engine, CL_SCAN_STDOPT);
+    fstat(fd, &sb);
+    if(!(*ctx.fmap = fmap(fd, 0, sb.st_size))) {
+	free(ctx.fmap);
+	cl_engine_free(engine);
+	return;
+    }
+    ret = cli_fmap_scandesc(&ctx, 0, 0, NULL, AC_SCAN_VIR, &acres, NULL);
+    res = acres;
+    while(res) {
+	matches++;
+	res = res->next;
+    }
+    if(matches) {
+	mprintf("MATCH: ** YES ** (%u %s:", matches, matches > 1 ? "matches at offsets" : "match at offset");
+	res = acres;
+	while(res) {
+	    mprintf(" %u", (unsigned int) res->offset);
+	    res = res->next;
+	}
+	mprintf(")\n");
+    } else {
+	mprintf("MATCH: ** NO **\n");
+    }
+    while(acres) {
+	res = acres;
+	acres = acres->next;
+	free(res);
+    }
+    free(ctx.fmap);
     cl_engine_free(engine);
-
-    return (ret == CL_VIRUS) ? 1 : 0;
 }
 
 static char *decodehexstr(const char *hex, unsigned int *dlen)
@@ -2189,6 +2281,14 @@ static int decodesig(char *sig, int fd)
 	const char *tokens[68];
 	int tokens_count, subsigs, i, bc = 0;
 
+    if(*sig == '[') {
+	if(!(pt = strchr(sig, ']'))) {
+	    mprintf("!decodesig: Invalid input\n");
+	    return -1;
+	}
+	sig = &pt[2];
+    }
+
     if(strchr(sig, ';')) { /* lsig */
         tokens_count = cli_strtokenize(sig, ';', 67 + 1, (const char **) tokens);
 	if(tokens_count < 4) {
@@ -2229,7 +2329,8 @@ static int decodesig(char *sig, int fd)
 		mprintf(" +-> DECODED SUBSIGNATURE:\n");
 		decodehex(pt ? pt : tokens[3 + i]);
 	    } else {
-		mprintf(" +-> MATCH: %s\n", matchsig(pt ? pt : tokens[3 + i], fd) ? "YES" : "** NO **");
+		mprintf(" +-> ");
+		matchsig(pt ? pt : tokens[3 + i], fd);
 	    }
 	}
     } else if(strchr(sig, ':')) { /* ndb */
@@ -2290,7 +2391,7 @@ static int decodesig(char *sig, int fd)
 	    mprintf("DECODED SIGNATURE:\n");
 	    decodehex(tokens[3]);
 	} else {
-	    mprintf("MATCH: %s\n", matchsig(tokens[3], fd) ? "YES" : "** NO **");
+	    matchsig(tokens[3], fd);
 	}
     } else if((pt = strchr(sig, '='))) {
 	*pt++ = 0;
@@ -2299,7 +2400,7 @@ static int decodesig(char *sig, int fd)
 	    mprintf("DECODED SIGNATURE:\n");
 	    decodehex(pt);
 	} else {
-	    mprintf("MATCH: %s\n", matchsig(pt, fd) ? "YES" : "** NO **");
+	    matchsig(pt, fd);
 	}
     } else {
 	mprintf("decodesig: Not supported signature format\n");
@@ -2336,11 +2437,6 @@ static int testsigs(const struct optstruct *opts)
 	return -1;
     }
 
-    if(cl_init(CL_INIT_DEFAULT) != CL_SUCCESS) {
-	mprintf("!testsigs: Can't initialize libclamav: %s\n", cl_strerror(ret));
-	return -1;
-    }
-
     sigs = fopen(optget(opts, "test-sigs")->strarg, "rb");
     if(!sigs) {
 	mprintf("!testsigs: Can't open file %s\n", optget(opts, "test-sigs")->strarg);
@@ -2570,6 +2666,7 @@ static void help(void)
     mprintf("    --build=NAME [cvd] -b NAME             build a CVD file\n");
     mprintf("    --no-cdiff                             Don't generate .cdiff file\n");
     mprintf("    --server=ADDR                          ClamAV Signing Service address\n");
+    mprintf("    --datadir=DIR				Use DIR as default database directory\n");
     mprintf("    --unpack=FILE          -u FILE         Unpack a CVD/CLD file\n");
     mprintf("    --unpack-current=SHORTNAME             Unpack local CVD/CLD into cwd\n");
     mprintf("    --list-sigs[=FILE]     -l[FILE]        List signature names\n");
@@ -2588,13 +2685,19 @@ static void help(void)
 
 int main(int argc, char **argv)
 {
-	int ret = 1;
+	int ret;
         struct optstruct *opts;
 	struct stat sb;
 
     if(check_flevel())
 	exit(1);
 
+    if((ret = cl_init(CL_INIT_DEFAULT)) != CL_SUCCESS) {
+	mprintf("!Can't initialize libclamav: %s\n", cl_strerror(ret));
+	return -1;
+    }
+    ret = 1;
+
     opts = optparse(NULL, argc, argv, 1, OPT_SIGTOOL, 0, NULL);
     if(!opts) {
 	mprintf("!Can't parse command line options\n");
