@@ -177,6 +177,9 @@ static const struct rtest {
 		      3 - blacklisted if 2nd db is loaded,
 		      4 - invalid regex*/
 } rtests[] = {
+        {NULL,"http://fake.example.com", "http://foo@key.com/", 0},
+	{NULL,"http://fake.example.com", "foo.example.com@key.com", 0},
+	{NULL,"http://fake.example.com", "foo@key.com", 2},
 	{NULL,"http://fake.example.com","&#61;&#61;&#61;&#61;&#61;key.com",0},
 	{NULL,"http://key.com","&#61;&#61;&#61;&#61;&#61;key.com",2},
 	{NULL," http://key.com","&#61;&#61;&#61;&#61;&#61;key.com",2},
@@ -298,7 +301,7 @@ static void psetup_impl(int load2)
 	f = fdopen(open_testfile("input/daily.pdb"),"r");
 	fail_unless(!!f, "fopen daily.pdb");
 
-	rc = load_regex_matcher(engine->domainlist_matcher,  f, &signo, 0, 0, NULL, 1);
+	rc = load_regex_matcher(engine, engine->domainlist_matcher,  f, &signo, 0, 0, NULL, 1);
 	fail_unless(rc == 0, "load_regex_matcher");
 	fclose(f);
 
@@ -309,7 +312,7 @@ static void psetup_impl(int load2)
 		fail_unless(!!f, "fopen daily.gdb");
 
 		signo = 0;
-		rc = load_regex_matcher(engine->domainlist_matcher,  f, &signo, 0, 0, NULL, 1);
+		rc = load_regex_matcher(engine, engine->domainlist_matcher,  f, &signo, 0, 0, NULL, 1);
 		fail_unless(rc == 0, "load_regex_matcher");
 		fclose(f);
 
@@ -322,7 +325,7 @@ static void psetup_impl(int load2)
 
 	f = fdopen(open_testfile("input/daily.wdb"),"r");
 	signo = 0;
-	rc = load_regex_matcher(engine->whitelist_matcher, f, &signo, 0, 1, NULL, 1);
+	rc = load_regex_matcher(engine, engine->whitelist_matcher, f, &signo, 0, 1, NULL, 1);
 	fail_unless(rc == 0,"load_regex_matcher");
 	fclose(f);
 
